/*fom232conky.c
  Gibt Sensorwerte und Luefterwerte des Fan-o-Matic Firmwareversion 2.32 aus
  und speichert die aktuellsten Werte in die Datei ~/fom.log
  Kompilieren bspw.: gcc -Wall -W -O3 -o fom232conky fom232conky2.c
  gebastelt vom neuling                                                    */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/signal.h>
#include <string.h>
#include <errno.h>

/* Angeblich arbeitet die Firmwareversion 2.32 mit dieser Baudrate: */
/*#define BAUDRATE B115200*/
/* Klappt aber nicht aber mit dieser funktioniert es: */
#define BAUDRATE B57600
#define FOM "/dev/ttyS1"
#define _POSIX_SOURCE 1

static int fd = 0; /* Filedescriptor fuer die serielle Schnittstelle */
static struct termios alt; /* alte serielle Porteinstellungen */
/* Signalmasken */
static sigset_t mask;
static sigset_t orig_mask;
static char modi[4] = {0}; /* Lueftermodi */
static char datenDa = 0; /* Sind Daten angekommen? */
static char lauf = 1; /* Soll Programm weiterlaufen? */

/* Signal Handler FOM oder Prog hat was gesendet */
static void signal_handler_IO(int status) {
	if(SIGIO == status) {
		datenDa = 1;
	} else if(SIGINT == status) {
		lauf = 0;
	}
}

/* Setzt den Signalhandler */
static void setzeSignalhandler() {
	struct sigaction saio;
	/* Signalhandler installieren */
	saio.sa_handler = signal_handler_IO;
	sigemptyset(&saio.sa_mask);
	saio.sa_flags = 0;
	saio.sa_restorer = NULL;
	sigaction(SIGIO,&saio,NULL);
	sigaction(SIGINT,&saio,NULL);
}

/* Verbindung zum FOM aufnehmen */
static void oeffnePort() {
	struct termios neu;

	/* Serielle Schnittstelle oeffnen */
	fd = open(FOM, O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK);
	/* Schauen obs gut gegangen ist */
	if(fd < 0) {
		perror(FOM);
		exit(EXIT_FAILURE);
	}

	/* Signalhandler installieren */
	setzeSignalhandler();

	 /* Dem SIGIO Signal erlauben diesen Prozess zu stoeren */
	fcntl(fd, F_SETOWN, getpid());
	/* File Descriptor asynchron setzen */
	fcntl(fd, F_SETFL, FASYNC);

	/* Porteinstellungen sichern bzw. termios intitialisieren */
	tcgetattr(fd,&alt);
	tcgetattr(fd,&neu);

	/* Kontrollflags setzen: */
	neu.c_cflag &= ~(CRTSCTS | CSTOPB | CSIZE | PARENB | PARODD | HUPCL);
	neu.c_cflag |= CLOCAL | CREAD | CS8;
	/* An FOM senden raw Modus */
	neu.c_oflag &= ~(OPOST | OLCUC | ONLCR | OCRNL);
	/* Vom FOM empfangen auch raw Modus */
	neu.c_lflag = ~(ICANON | ECHO | ECHOE | ISIG);
	/* Einstellen wie die ankommenden Daten vom FOM verarbeitet werden */
	neu.c_iflag &= ~(BRKINT | ICRNL | IGNBRK | IGNCR | IMAXBEL | INLCR |
			INPCK | ISTRIP | IUCLC | IXOFF | IXON | PARMRK);
	neu.c_iflag |= IGNPAR;

	/* Baudrate fuer Ein- und Ausgang setzen */
	cfsetospeed(&neu, BAUDRATE);
	cfsetispeed(&neu, BAUDRATE);

	/* Ein und Ausgabebuffer vom FOM loeschen */
	tcflush(fd, TCIOFLUSH);

	/* Die Einstellungen an den File Desciptor uebergeben */ 
	tcsetattr(fd,TCSANOW,&neu);
}

/* Fragt die Konfigurationsdaten ab und speichert die Lueftermodi ab */
static void frageCfgAb() {
	int i = 0;
	int g = 0;
	int zaehler = 0;
	int versuche = 0;
	char buf[128] = {0};
	char buf2[128] = {0};
	unsigned char hk[5] = {0x55, 0x09, 0x05, 0x11, 0x00};

	/* 10 Versuche die Konfigurationsdaten abzufragen */
	for (versuche = 0; versuche < 10; versuche++) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		if (-1 == write(fd,hk,5)) {
			perror("write");
			lauf = 0;
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		/* Auf Antwort vom FOM warten und ausgeben */
		while (lauf) {
			if (datenDa) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				g = (int)read(fd,buf,32);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = 0;
			}
			if (97 < zaehler) {
				/* Luefterkonfiguration abspeichern */
				for (i = 0; i < 4; i++) {
					modi[i] = buf2[4 + i * 14];
				}
				return;
			}
			usleep(250000);
			if (!datenDa) {
				zaehler = 0;
				break;
			}
		}
	}
	fprintf(stderr, "Firmwareversion konnte nicht abgefragt werden!");
	lauf = 0;
}

int main(void) {
	int i = 0;
	int g = 0;
	int zaehler = 0;
	FILE *fomlog = NULL; /* Filezeiger auf die Datei fom.log */
	char pfadname[128] = {0};
	char buf[256] = {0};
	char buf2[256] = {0};
	/* Anfragefolge fuer Daten vom Fom */
	const unsigned char ds[5] = {0x55,0x09,0x05,0x13,0x00};
	char dfs = 0;

	/* Signale sollten zwar read nicht aus dem Konzept
	   bringen aber sicher ist sicher                   */         
	sigemptyset(&mask);
	sigaddset(&mask, SIGIO);
	sigaddset(&mask, SIGINT);

	/* Versuch mit dem FOM Verbindung aufzunehmen */
	oeffnePort();

	/* Konfigurationsdaten abfragen */
	frageCfgAb();

	/* pfadnamen zusammenbasteln */
	snprintf(pfadname, 128, "%s/fom.log", getenv("HOME"));

	/* Programm beenden mit SIGINT oder SIGTERM */
	while (lauf) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		if (-1 == write(fd,ds,5)) {
			perror("write");
			lauf = 0;
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		while (lauf) {
			if (datenDa) {
				/* Bytes lesen und ausgeben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				g = (int)read(fd,buf,128);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = 0;
				if (25 < zaehler) {
					/* fom.log Datei erstellen oder oeffnen */
					if ( NULL == (fomlog = fopen(pfadname, "w")) ) {
						perror("fopen fom.log");
						lauf = 0;
						break;
					}
					/* Lüfter ausgeben */
					fprintf(fomlog, "Lüfter: ");
					for (g = 0; g < 4; g++) {
						if (modi[g] < 3) {
							if (0 < ((buf2[12] >> g) & 0x01)) {
								fprintf(fomlog, "${color1}%2d00 rpm %2d.%dV",
									buf2[8 + g],
									buf2[4 + g] / 10,
									buf2[4 + g] % 10);
								if (3 != g && 3 > modi[g + 1]
									&& 0 < ((buf2[12] >> (g + 1)) & 0x01)) {
									fprintf(fomlog, "${color0}, ");
								} else {
									fprintf(fomlog, " ");
								}
							}
						} else {
							/* Durchflussausgabe anstellen */
							dfs = 1;
						}
					}
					/* Durchfluss ausgeben */
					if (dfs) {
						fprintf(fomlog, "${color0}DFM:${color1}%3d l/h ", buf2[22]);
					}
					/* Sensoren ausgeben */
					fprintf(fomlog, "${color0}${font weather:size=11}x${font Impact:size=10}${color1}");
					for (g = 0; g < 6; g++) {
						fprintf(fomlog, "%3d", buf2[13 + g]);
						if (5 != g) {
							fprintf(fomlog, " ");
						}
					}
					/* Datei fom.log schliessen */
					fclose(fomlog);
					usleep(1000000);
					break;
				}
			}
			/* Eine Viertelsekunde pennen in dieser Zeit sollte der FOM
			   der Sendeaufforderung nachgekommen sein. Wenn nicht wird
			   die Sendeaufforderung erneut gesendet.
			   Das Gepenne wird durch das SIGIO Signal unterbrochen */
			usleep(250000);
			if (!datenDa) {
				break;
			}
		}
		zaehler = 0;
	}
	/* Porteinstellungen zurueckschreiben */
	tcsetattr(fd,TCSANOW,&alt);

	/* Serielle Schnittstelle wieder schliessen */
	close(fd);

	return EXIT_SUCCESS;
}
