/*fom232nc.c
  Gibt Sensorwerte und Luefterwerte des Fan-o-Matic Firmwareversion 2.32 aus
  Kompilieren bspw.: gcc -Wall -W -O3 -o fom232nc fom232nc.c -lncursesw -pthread
  Programmiert von Johannes Ullmann
  Nick innovatek Forum: neuling  Nick hardwareluxx Forum: neuli               */

#include <ncursesw/curses.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/signal.h>
#include <sys/ioctl.h>
#include <string.h>
#include <locale.h>
#include <langinfo.h>
#include <pthread.h>

/* Angeblich arbeitet die Firmwareversion 2.32 mit dieser Baudrate: */
/*#define BAUDRATE B115200*/
/* Klappt aber nicht aber mit dieser funktioniert es: */
#define BAUDRATE B57600
#define FOM "/dev/ttyS1"
#define _POSIX_SOURCE 1

static int fd = 0; /* Filedescriptor fuer die serielle Schnittstelle */
static WINDOW *links = NULL; /* Linkes Fenster */
static WINDOW *rechts = NULL; /* Rechtes Fenster */
static WINDOW *unten = NULL; /* unteres Fenster */
static WINDOW *haupt = NULL; /* Das Haupt Fenster */
static pthread_mutex_t mutex; /* wird zum Sperren von weiter verwendet */
static pthread_mutex_t mutex2; /* wird zum Sperren von lauf verwendet */
static pthread_mutex_t mutex3; /* synchronisiert die ncurses Ausgabe */
static pthread_mutex_t mutex4; /* syncronisiert den FOM Datenverkehr */
static struct termios alt; /* alte serielle Porteinstellungen */
/* Signalmasken */
static sigset_t mask;
static sigset_t orig_mask;
static sigset_t mask2;
static sigset_t orig_mask2;
static unsigned short pulse = 0; /* Eingestellte DFS Pulsrate */
static bool datenDa = FALSE; /* Sind Daten angekommen? */
static bool lauf = TRUE; /* Soll Programm weiterlaufen? */
static bool fensterNeu = FALSE; /* muss Fenster angepasst werden? */
static bool weiter = FALSE; /* muss Modus gewechselt werden? */
static char namen[10][21] = {{0}}; /* Namen der Luefter und Sensoren */
static char meld[10] = {0}; /* Alarm Meldung der Luefter und Sensoren */
static char mobo[10] = {0}; /* Alarm Mainboard der Luefter und Sensoren */
static char relais[10] = {0}; /* Alarm Relaisausgang der Luefter und Sensoren */
static char aus12V[10] = {0}; /* Alarm 12V Ausgang der Luefter und Sensoren */
static char version[6] = {0}; /* FOM Firmwareversion */
static char solltemp[6] = {0}; /* Solltemperatur der Sensoren */
static char dfs[4] = {0}; /* Alarmmeldungen vom Durchflussensor */
static char modi[4] = {0}; /* Lueftermodi */
static char sensoren[4] = {0}; /* Mit welchem Sensor der Luefter geregelt wird */
static char lp[4] = {0}; /* Wieviele Impulse geben eine Lüfterdrehung */
static char min[4] = {0}; /* Mindestspannung der Luefter */
static char max[4] = {0}; /* Maximalspannung der Luefter */
static char boost[4] = {0}; /* Boostspannung der Luefter */
static char fo[4] = {0}; /* Fan-Off Automatik darf Luefter abschalten */
static char zeit[4] = {0}; /* Reglungsinterval in Sekunden */
static char ig[4] = {0}; /* Reglung Integral */
static char prop[4] = {0}; /* Reglung Proportional */
static char span[4] = {0}; /* Spannung der Luefter */
static char lz[4] = {0}; /* Luefter an oder aus */
static char dto = 0; /* Display Timeout in Minuten */
static unsigned char menu = 0; /* Welcher Menueeintrag ist ausgewaehlt? */
static unsigned char aktiv = 0; /* Welcher Modus ist aktiv? */

/* Ein Fehlerfall ist aufgetreten */
static void gingNichtGut(const char* nachricht) {
	/* Porteinstellungen zurueckschreiben */
	tcsetattr(fd,TCSANOW,&alt);
	/* Serielle Schnittstelle wieder schliessen */
	close(fd);
	/* eventuelles links Fenster schliessen */
	if (NULL != links) {
		delwin(links);
	}
	/* eventuelles rechts Fenster schliessen */
	if (NULL != rechts) {
		delwin(rechts);
	}
	/* eventuelles unteres Fenster schliessen */
	if (NULL != unten) {
		delwin(unten);
	}
	/* curses mode beenden */
	endwin();
	/* Fehlermeldung ausgeben */
	printf("%s\n", nachricht);
	/* Programm beenden */
	abort();
}

/* Curses Modus starten */
static void starteCurses() {
	/* curses Modus starten */
	haupt = initscr();
	/* Bildschirm leeren */
	clear();
	/* abfragen, ob Terminal Farben unterstuetzt */
	if (has_colors() == FALSE) {
		gingNichtGut("Das Terminal unterstuetzt keine Farben!");
	}
	/* Farbmodus aktivieren */
	start_color();
	/* Farbpaar 1 definieren */
	init_pair(1, COLOR_BLUE, COLOR_WHITE);
	/* Farbpaar 2 definieren */
	init_pair(2, COLOR_BLUE, COLOR_BLACK);
	/* Farbpaar 3 definieren */
	init_pair(3, COLOR_CYAN, COLOR_BLACK);
	/* Farbpaar 4 definieren */
	init_pair(4, COLOR_RED, COLOR_BLACK);
	/* Farbpaar 5 definieren */
	init_pair(5, COLOR_BLACK, COLOR_GREEN);
	/* Farbpaar 6 definieren */
	init_pair(6, COLOR_BLACK, COLOR_RED);
	/* raw Modus aber Signale weiterleiten! */
	cbreak();
	/* F1, F2 etc ... */
	keypad(haupt, TRUE);
	/* auf Tasteneingaben warten */
	nodelay(haupt, FALSE);
	/* Keine Ausgabe beim Holen von Zeichen */
	noecho();
	/* Cursor unsichtbar setzen */
	curs_set(0);
	/* Baue linkes Fenster */
	links = newwin(LINES -3, (COLS - 0.5)/ 2 - 1, 2, 1);
	/* keypad fuer Fenster */
	keypad(links, TRUE);
	/* auf Tasteneingaben warten */
	nodelay(links, FALSE);
	/* Dieses Fenster benoetigen nur zwei Modi */
	if (2 != aktiv) {
		/* Baue rechtes Fenster */
		rechts = newwin(LINES -3, (COLS - 0.5)/ 2 - 1, 2, COLS / 2);
		/* keypad fuer rechtes Fenster */
		keypad(rechts, TRUE);
		/* auf Tasteneingaben warten */
		nodelay(rechts, FALSE);
	}
}

/* Umrahmung malen */
static void maleRahmen(int hoehe, int breite) {
	int i = 0;
	int tmp = 0;
	attron(COLOR_PAIR(2));
	mvwaddch(haupt, 0, 0, ACS_ULCORNER);
	if (26 < getmaxx(haupt)) {
		tmp = breite / 2 - 13;
	} else {
		tmp = breite / 2 - 7;
	}
	for (i = 1; i < tmp; i++) {
		waddch(haupt, ACS_HLINE);
	}
	attroff(COLOR_PAIR(2));
	if (26 < getmaxx(haupt)) {
		wprintw(haupt, "FOM Firmwareversion: %s", version);
		tmp += 26;
	} else {
		wprintw(haupt, "FOM-Ver.: %s", version);
		tmp += 14;
	}
	attron(COLOR_PAIR(2));
	for (i = tmp; i < breite; i++) {
		waddch(haupt, ACS_HLINE);
	}
	waddch(haupt, ACS_URCORNER);
	for (i = 1; i < hoehe - 1; i++) {
		mvwaddch(haupt, i, 0, ACS_VLINE);
		mvwaddch(haupt, i, breite - 1, ACS_VLINE);
	}
	mvwaddch(haupt, hoehe - 1, 0, ACS_LLCORNER);
	for (i = 1; i < breite - 1; i++) {
		waddch(haupt, ACS_HLINE);
	}
	waddch(haupt, ACS_LRCORNER);
}

/* Berechnet wie viele automatische Zeilenumbrueche
   das Konfigurationsmenue braucht                  */
static int berechneZeilenumbrueche() {
	/* Es reicht nur die linke Seite zu berechnen, da sie immer
	   groesser oder gleich der rechten Seite ist               */
	int tmp = 35; /* erste Zeile */
	tmp += 44; /* zweite Zeile */
	tmp += 43; /* dritte Zeile */
	tmp += 53; /* vierte Zeile */
	tmp += 49; /* fuenfte Zeile */
	return tmp / ((COLS - 0.5) / 2 - 1);
}

/* Fenster anpassen */
static void passeFensterAn() {
	int tmp = 0;
	struct winsize groessen;
	if (-1 == ioctl(fileno (stdout), TIOCGWINSZ, &groessen) ) {
		gingNichtGut("Fenstergroesse konnte nicht angepasst werden");
	}
	keypad(links, FALSE);
	/* dieses Fenster ist immer vorhanden */
	delwin(links);
	if (NULL != rechts) {
		keypad(rechts, FALSE);
		delwin(rechts);
		rechts = NULL;
	}
	if (NULL != unten) {
		keypad(unten, FALSE);
		delwin(unten);
		unten = NULL;
	}
	keypad(haupt, FALSE);
	endwin();
	resizeterm(groessen.ws_row,groessen.ws_col);
	starteCurses();
	wresize(haupt, groessen.ws_row, groessen.ws_col);
	/* Nicht jeder Modus braucht zwei Fenster */
	if (2 == aktiv) {
		wresize(links, 3, groessen.ws_col - 2 );
		unten = newwin(LINES - 6, COLS - 2, 5, 1);
		wresize(unten, groessen.ws_row - 6, groessen.ws_col - 2);
	} else {
		wresize(links, groessen.ws_row - 3, ((groessen.ws_col - 0.5) / 2 - 1));
		wresize(rechts, groessen.ws_row - 3, ((groessen.ws_col - 0.5) / 2 - 1));
		if (1 == aktiv) {
			tmp = berechneZeilenumbrueche();
			unten = newwin(LINES - 9 - tmp, COLS - 3, 8 + tmp, 1);
			wresize(unten, groessen.ws_row - 9 - tmp, groessen.ws_col - 3);
			wrefresh(unten);
		}
		wrefresh(rechts);
	}
	wrefresh(links);
	maleRahmen(getmaxy(haupt), getmaxx(haupt));
	wrefresh(haupt);
	fensterNeu = FALSE;
}

/* Signal Handler FOM oder Prog hat was gesendet */
static void signal_handler_IO (int status) {
	if(SIGIO == status) {
		datenDa = TRUE;
	} else if(SIGINT == status) {
		lauf = FALSE;
	} else if(SIGWINCH == status) {
		fensterNeu = TRUE;
	}
}

/* Setzt den Signalhandler */
static void setzeSignalhandler() {
	struct sigaction saio;
	/* Signalhandler installieren */
	saio.sa_handler = signal_handler_IO;
	sigemptyset(&saio.sa_mask);
	saio.sa_flags = 0;
	saio.sa_restorer = NULL;
	sigaction(SIGIO,&saio,NULL);
	sigaction(SIGINT,&saio,NULL);
	sigaction(SIGWINCH,&saio,NULL);
}

/* Verbindung zum FOM aufnehmen */
static void oeffnePort() {
	/* uninitialisiert */
	struct termios neu;

	/* Serielle Schnittstelle oeffnen */
	fd = open(FOM, O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK);
	/* Schauen obs gut gegangen ist */
	if (fd < 0) {
		perror(FOM);
		abort();
	}

	/* Signalhandler installieren */
	setzeSignalhandler();

	 /* Dem SIGIO Signal erlauben diesen Prozess zu stoeren */
	fcntl(fd, F_SETOWN, getpid());
	/* File Descriptor asynchron setzen */
	fcntl(fd, F_SETFL, FASYNC);

	/* Porteinstellungen sichern bzw. termios intitialisieren */
	tcgetattr(fd,&alt);
	tcgetattr(fd,&neu);

	/* Kontrollflags setzen: */
	neu.c_cflag &= ~(CRTSCTS | CSTOPB | CSIZE | PARENB | PARODD | HUPCL);
	neu.c_cflag |= CLOCAL | CREAD | CS8;
	/* An FOM senden raw Modus */
	neu.c_oflag &= ~(OPOST | OLCUC | ONLCR | OCRNL);
	/* Vom FOM empfangen auch raw Modus */
	neu.c_lflag &= ~(ECHO | ECHOE | ICANON | ISIG);
	/* Einstellen wie die ankommenden Daten vom FOM verarbeitet werden */
	neu.c_iflag &= ~(BRKINT | ICRNL | IGNBRK | IGNCR | IMAXBEL | INLCR |
			INPCK | ISTRIP | IUCLC | IXOFF | IXON | PARMRK);
	neu.c_iflag |= IGNPAR;

	/* Baudrate fuer Ein- und Ausgang setzen */
	cfsetospeed(&neu, BAUDRATE);
	cfsetispeed(&neu, BAUDRATE);

	/* Ein und Ausgabebuffer vom FOM loeschen */
	tcflush(fd, TCIOFLUSH);

	/* Die Einstellungen an den File Desciptor uebergeben */
	tcsetattr(fd, TCSANOW, &neu);
}

/* FOM Firmwareversion abfragen und in version speichern */
static void gebeIDaus() {
	ssize_t g = 0;
	/* buffer fuer read */
	char buf[16] = {0};
	char buf2[16] = {0};
	/* Bytefolge zum Abfragen der Firmwarversion */
	const unsigned char vp[5] = {0x55,0xFF,0x05,0x01,0x00};
	unsigned char zaehler = 0;
	unsigned char i = 0;
	unsigned char versuche = 0;

	/* 10 Versuche die Firmwareversion abzufragen */
	for (versuche = 0; versuche < 10; versuche++) {
		/* FOM Firmwareverion abfragen */
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		if (-1 == write(fd,vp,5)) {
			gingNichtGut("write schlug fehl!");
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		/* Auf Antwort vom FOM warten und ausgeben */
		while (lauf) {
			if (datenDa) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				g = read(fd,buf,16);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = FALSE;
			}
			if (4 < zaehler) {
				snprintf(version, 5, "%d.%d",
						 (buf2[4] & 0xFF) / 100, (buf2[4] & 0xFF) % 100);
				return;
			}
			usleep(250000);
			if (!datenDa) {
				zaehler = 0;
				break;
			}
		}
	}

	if (lauf) {
		gingNichtGut("Firmwareversion konnte nicht abgefragt werden!");
	}
}

/* Fragt die Konfigurationsdaten ab und speichert diese im Heap */
static void frageCfgAb() {
	ssize_t g = 0;
	char buf[128] = {0};
	char buf2[128] = {0};
	unsigned char hk[5] = {0x55, 0x09, 0x05, 0x11, 0x00};
	unsigned char versuche = 0;
	unsigned char zaehler = 0;
	unsigned char i = 0;

	/* 10 Versuche die Konfigurationsdaten abzufragen */
	for (versuche = 0; versuche < 10; versuche++) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		if (-1 == write(fd,hk,5)) {
			gingNichtGut("write schlug fehl!");
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		/* Auf Antwort vom FOM warten und ausgeben */
		while (lauf) {
			if (datenDa) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				g = read(fd,buf,32);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = FALSE;
			}
			if (97 < zaehler) {
				/* Luefterkonfiguration abspeichern */
				for (i = 0; i < 4; i++) {
					modi[i] = buf2[4 + i * 14];
					meld[i] = buf2[5 + i * 14];
					mobo[i] = buf2[6 + i * 14];
					relais[i] = buf2[7 + i * 14];
					aus12V[i] = buf2[8 + i * 14];
					sensoren[i] = buf2[9 + i * 14];
					lp[i] = buf2[10 + i * 14];
					min[i] = buf2[11 + i * 14];
					max[i] = buf2[12 + i * 14];
					boost[i] = buf2[13 + i * 14];
					fo[i] = buf2[14 + i * 14];
					zeit[i] = buf2[15 + i * 14];
					ig[i] = buf2[16 + i * 14];
					prop[i] = buf2[17 + i * 14];
					dfs[i] = buf2[90 + i];
				}
				/* Sensorenkonfigurtaion abspeichern */
				for (i = 0; i < 6; i++) {
					solltemp[i] = buf2[60 + i * 5];
					meld[i + 4] = buf2[61 + i * 5];
					mobo[i + 4] = buf2[62 + i * 5];
					relais[i + 4] = buf2[63 + i * 5];
					aus12V[i + 4] = buf2[64 + i * 5];
				}
				pulse = buf2[96] << 8;
				pulse |= buf2[95] & 0xFF;
				dto = buf2[97];
				return;
			}
			usleep(250000);
			if (!datenDa) {
				zaehler = 0;
				break;
			}
		}
	}
	if (lauf) {
		gingNichtGut("Konfigurationsdaten konnten nicht abgefragt werden!");
	}
}

/* Fragt den Namen des Luefters oder Sensors ab
   und speichert diesen im Feld namen           */
static void holeNamen(unsigned char nummer) {
	ssize_t g = 0;
	char buf[32] = {0};
	char buf2[32] = {0};
	unsigned char hn[6] = {0x55, 0x09, 0x06, 0x51, 0x00, 0x82};
	unsigned char versuche = 0;
	unsigned char i = 0;
	unsigned char zaehler = 0;

	hn[5] += nummer;
	/* 10 Versuche den Namen abzufragen */
	for (versuche = 0; versuche < 10; versuche++) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		if (-1 == write(fd,hn,6)) {
			gingNichtGut("write schlug fehl!");
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		/* Auf Antwort vom FOM warten und ausgeben */
		while (lauf) {
			if (datenDa) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				g = (int)read(fd,buf,32);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = FALSE;
			}
			if (25 < zaehler) {
				strncpy(namen[nummer], &buf2[6], 20);
				namen[nummer][20] = '\0';
				return;
			}
			usleep(250000);
			if (!datenDa) {
				zaehler = 0;
				break;
			}
		}
	}
	gingNichtGut("Die Lüfter und Sensornamen konnten nicht abgefragt werden!");
}

/* Elende Umlaute und Sonderzeichen von utf8 erkennen und fuer FOM umwandeln */
static unsigned char ausZweiMachEins(int zeichen1, int zeichen2,
				int zeile, int spalte, WINDOW *wohin) {
	/* Wird erweitert wenn ich ein neues Sonderzeichen brauch */
	switch (zeichen1) {
		case 194:
			switch (zeichen2) {
				case 176:
					mvwprintw(wohin, zeile, spalte, "°");
					return 0xDF;
			}
			break;
		case 195:
			switch (zeichen2) {
				case 164:
					mvwaddstr(wohin, zeile, spalte, "ä");
					return 0xE1;
				case 182:
					mvwaddstr(wohin, zeile, spalte, "ö");
					return 0xEF;
				case 188:
					mvwaddstr(wohin, zeile, spalte, "ü");
					return 0xF5;
				case 132:
					mvwaddstr(wohin, zeile, spalte, "Ä");
					/* Code im FOM noch unbekannt */
					return 0xE1;
				case 150:
					mvwaddstr(wohin, zeile, spalte, "Ö");
					/* Code im FOM noch unbekannt */
					return 0xEF;
				case 156:
					mvwaddstr(wohin, zeile, spalte, "Ü");
					/* Code im FOM noch unbekannt */
					return 0xF5;
				case 159:
					mvwaddstr(wohin, zeile, spalte, "ß");
					return 0xE2;
			}
			break;
	}
	return ' ';
}

/* Elende Umlaute und Sonderzeichen vom FOM erkennen und fuer utf8 umwandeln */
static void ausEinsMachZwei(unsigned char zeichen, WINDOW *wohin) {
	switch (zeichen) {
		case 0xE1:
			waddstr(wohin, "ä");
			break;
		case 0xEF:
			waddstr(wohin, "ö");
			break;
		case 0xF5:
			waddstr(wohin, "ü");
			break;
		case 0xDF:
			waddstr(wohin, "°");
			break;
		case 0xE2:
			waddstr(wohin, "ß");
			break;
		default:
			waddch(wohin,zeichen);
	}
}

/* Oberes Menue ausgeben */
static void starteMenue() {
	const char *menue[] = { "Übersicht", "Konfiguration", "Texttool" };
	unsigned char i = 0;

	/* Menuezeile bereinigen */
	wmove(haupt, 1, 1);
	for (i = 1; i < getmaxx(haupt) - 1; i++) {
		waddch(haupt, ' ');
	}

	/* Farbpaar 1 aktivieren */
	attron(COLOR_PAIR(1));
	wmove(haupt, 1, COLS / 2 - 15);
	/* Menue ausgeben */
	for (i = 0; i < 3; i++) {
		if (menu == i) {
			attron(A_REVERSE);
			wprintw(haupt, "%s", menue[i]);
			attroff(A_REVERSE);
		} else {
			wprintw(haupt, "%s", menue[i]);
		}
		if (3 > i) {
			attroff(COLOR_PAIR(1));
			waddch(haupt, ' ');
			attron(COLOR_PAIR(1));
		}
	}
	/* Farbpaar 1 deaktivieren */
	attroff(COLOR_PAIR(1));
	wrefresh(haupt);
}

/* Tastenabfrage wenn im oberen Menue */
static bool frageTasteAb() {
	int eingabe = 0;
	bool rueck = FALSE;
	eingabe = wgetch(haupt);
	switch (eingabe) {
		case ERR:
			rueck = FALSE;
			break;
		case KEY_LEFT:
			menu = (3 + menu - 1) % 3;
			pthread_mutex_lock(&mutex3);
			starteMenue();
			pthread_mutex_unlock(&mutex3);
			rueck = FALSE;
			break;
		case KEY_RIGHT:
			menu = (menu + 1) % 3;
			pthread_mutex_lock(&mutex3);
			starteMenue();
			pthread_mutex_unlock(&mutex3);
			rueck = FALSE;
			break;
		case (int)'\n':
			aktiv = menu;
			rueck = TRUE;
			break;
		case (int)'q':
			pthread_mutex_lock(&mutex2);
			lauf = FALSE;
			pthread_mutex_unlock(&mutex2);
			rueck = TRUE;
			break;
			/* Auskommentieren um zu schauen welche Codes welche Tasten haben */
			/*default:
				printw("Tasteneingabe: %d ", eingabe);*/
	}
	return rueck;
}

/* Fuer Faden 2 ruft solange frageTasteAb auf bis Modus gewechselt wird */
static void *menueWechseln(void *data) {
	/* Im anderen Faden werden die Signale ausgeloest */
	sigprocmask(SIG_BLOCK,&mask,&orig_mask);
	while(TRUE) {
		pthread_mutex_lock(&mutex2);
		if (!lauf) {
			pthread_mutex_unlock(&mutex2);
			break;
		}
		pthread_mutex_unlock(&mutex2);
		if (frageTasteAb()) {
			/* Den anderen Thread daran hindern auf weiter zuzugreifen */
			pthread_mutex_lock(&mutex);
			weiter = TRUE;
			pthread_mutex_unlock(&mutex);
			break;
		}
	}
	sigprocmask(SIG_SETMASK,&orig_mask,NULL);
	return data;
}

/* Hilfsfunktion gebeUebersichtAus: Alarmmedlungsausgabe */
static void gebeAlarmmeldungenAus(char * buf2) {
 	unsigned char i = 0;
	wmove(haupt, LINES - 1, COLS / 2 - 27);
	for (i = 0; i < 4; i++) {
		if ((buf2[23] >> i) & 0x01) {
			wattron(haupt, COLOR_PAIR(6));
			wprintw(haupt, "l%d", i + 1);
			wattroff(haupt, COLOR_PAIR(6));
		} else {
			wattron(haupt, COLOR_PAIR(5));
			wprintw(haupt, "l%d", i + 1);
			wattroff(haupt, COLOR_PAIR(5));
		}
		wattron(haupt, COLOR_PAIR(2));
		waddch(haupt, ACS_HLINE);
		wattroff(haupt, COLOR_PAIR(2));
	}
	if ((buf2[23] >> i) & 0x10) {
		wattron(haupt, COLOR_PAIR(6));
		waddstr(haupt, "DFM");
		wattroff(haupt, COLOR_PAIR(6));
	} else {
		wattron(haupt, COLOR_PAIR(5));
		waddstr(haupt, "DFM");
		wattroff(haupt, COLOR_PAIR(5));
	}
	wattron(haupt, COLOR_PAIR(2));
	waddch(haupt, ACS_HLINE);
	wattroff(haupt, COLOR_PAIR(2));
	for (i = 0; i < 6; i++) {
		if ((buf2[24] >> i) & 0x01) {
			wattron(haupt, COLOR_PAIR(6));
			wprintw(haupt, "s%d", i + 1);
			wattroff(haupt, COLOR_PAIR(6));
		} else {
			wattron(haupt, COLOR_PAIR(5));
			wprintw(haupt, "s%d", i + 1);
			wattroff(haupt, COLOR_PAIR(5));
		}
		wattron(haupt, COLOR_PAIR(2));
		waddch(haupt, ACS_HLINE);
		wattroff(haupt, COLOR_PAIR(2));
	}
	if (buf2[25] & 0x01) {
		wattron(haupt, COLOR_PAIR(6));
		waddstr(haupt, "Meldung");
		wattroff(haupt, COLOR_PAIR(6));
		beep();
	} else {
		wattron(haupt, COLOR_PAIR(5));
		waddstr(haupt, "Meldung");
		wattroff(haupt, COLOR_PAIR(5));
	}
	wattron(haupt, COLOR_PAIR(2));
	waddch(haupt, ACS_HLINE);
	wattroff(haupt, COLOR_PAIR(2));
	if (buf2[25] & 0x02) {
		wattron(haupt, COLOR_PAIR(5));
		waddstr(haupt, "Mobo");
		wattroff(haupt, COLOR_PAIR(5));
	} else {
		wattron(haupt, COLOR_PAIR(6));
		waddstr(haupt, "Mobo");
		wattroff(haupt, COLOR_PAIR(6));
	}
	wattron(haupt, COLOR_PAIR(2));
	waddch(haupt, ACS_HLINE);
	wattroff(haupt, COLOR_PAIR(2));
	if (buf2[25] & 0x08) {
		wattron(haupt, COLOR_PAIR(6));
		waddstr(haupt, "Relais");
		wattroff(haupt, COLOR_PAIR(6));
	} else {
		wattron(haupt, COLOR_PAIR(5));
		waddstr(haupt, "Relais");
		wattroff(haupt, COLOR_PAIR(5));
	}
	wattron(haupt, COLOR_PAIR(2));
	waddch(haupt, ACS_HLINE);
	wattroff(haupt, COLOR_PAIR(2));
	if (buf2[25] & 0x04) {
		wattron(haupt, COLOR_PAIR(6));
		waddstr(haupt, "12V");
		wattroff(haupt, COLOR_PAIR(6));
	} else {
		wattron(haupt, COLOR_PAIR(5));
		waddstr(haupt, "12V");
		wattroff(haupt, COLOR_PAIR(5));
	}
	wrefresh(haupt);
}

/* Gibt alle FOMdaten aus und zeigt diese an */
static void gebeUebersichtAus() {
	ssize_t g = 0;
	pthread_t thread = 0;
	FILE *fomlog = NULL; /* Filezeiger auf die Datei fom.log */
	char pfadname[128] = {0};
	char buf[256] = {0};
	char buf2[256] = {0};
	/* Anfragefolge fuer Daten vom Fom */
	const unsigned char ds[5] = {0x55,0x09,0x05,0x13,0x00};
	const char *laa[] = {"aus", "an"};
	const char *lm[] = {"Manuell", "Automatik",
			"Flow-O-Matic", "Flow-Meter"};
	unsigned char i = 0;
	unsigned char zaehler = 0;

	/* pfadnamen zusammenbasteln */
	snprintf(pfadname, 128, "%s/fom.log", getenv("HOME"));

	pthread_create(&thread, NULL, menueWechseln, NULL);
	/* Programm beenden mit SIGINT oder SIGTERM */
	while (TRUE) {
		pthread_mutex_lock(&mutex);
		pthread_mutex_lock(&mutex2);
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		if (!lauf || weiter) {
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			pthread_mutex_unlock(&mutex2);
			pthread_mutex_unlock(&mutex);
			pthread_join(thread, NULL);
			return;
		}
		pthread_mutex_unlock(&mutex2);
		pthread_mutex_unlock(&mutex);
		if (-1 == write(fd,ds,5)) {
			pthread_mutex_lock(&mutex2);
			lauf = FALSE;
			pthread_mutex_unlock(&mutex2);
			pthread_join(thread, NULL);
			gingNichtGut("write schlug fehl!");
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		while (TRUE) {
			pthread_mutex_lock(&mutex2);
			sigprocmask(SIG_BLOCK,&mask,&orig_mask);
			if (!lauf) {
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				pthread_mutex_unlock(&mutex2);
				pthread_join(thread, NULL);
				return;
			}
			pthread_mutex_unlock(&mutex2);
			/* Nur dieser Faden passt das Fenster an */
			pthread_mutex_lock(&mutex3);
			if (fensterNeu) {
				passeFensterAn();
				starteMenue();
			}
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			pthread_mutex_unlock(&mutex3);

			if (datenDa) {
				pthread_mutex_lock(&mutex3);
				wmove(links, 0, 0);
				wmove(rechts, 0, 0);
				pthread_mutex_unlock(&mutex3);
				/* Bytes lesen und ausgeben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				g = read(fd,buf,128);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = FALSE;
				if (25 < zaehler) {
					/* fom.log Datei erstellen oder oeffnen */
					if ( NULL == (fomlog = fopen(pfadname, "w")) ) {
						perror("fopen fom.log");
						pthread_mutex_lock(&mutex2);
						lauf = FALSE;
						pthread_mutex_unlock(&mutex2);
						break;
					}
					pthread_mutex_lock(&mutex3);
					wclear(links);
					wclear(rechts);
					/* Luefter ausgeben */
					fprintf(fomlog, "Lüfter: ");
					for (g = 0; g < 4; g++) {
						if (1 == (g & 1)) {
							wattron(links, COLOR_PAIR(3));
						}
						wprintw(links, "Lüfter%d ist %s Name: ",
								g + 1, laa[(buf2[12] >> g) & 0x01]);
						for (i = 0; i < 20; i++) {
							ausEinsMachZwei(namen[g][i], links);
						}
						wprintw(links, "\n%2d00 rpm "
								"%2d.%dV min: %2d.%d max: %2d.%d Boost: %2d.%d "
								"%s\nSensor%d Fan-Off: %s Zeit: %d Integral:"
								"%2d.%d Proportional:%2d.%d\nMeldung: %2d00 "
								"Mobo: %2d00 Relais: %2d00 12v: %2d00\n",
								buf2[8 + g], (unsigned int)buf2[4 + g] / 10,
								(unsigned int)buf2[4 + g] % 10,
								min[g] / 10, min[g] % 10, max[g] / 10,
								max[g] % 10, boost[g] / 10, boost[g] % 10,
								lm[(int)modi[g]], sensoren[g], laa[(int)fo[g]],
								zeit[g], ig[g] / 10, ig[g] % 10, prop[g] / 10,
								prop[g] % 10,
								meld[g], mobo[g], relais[g], aus12V[g]);
						if (1 == (g & 1)) {
							wattroff(links, COLOR_PAIR(3));
						}
						/* Aktuelle Spannung und
						   Luefter an oder aus speichern */
						span[g] = buf2[4 + g];
						lz[g] = (buf2[12] >> g) & 0x01;
						i = 0;
						if (modi[g] < 3) {
							if (0 < lz[g]) {
								fprintf(fomlog, "${color1}%2d00 rpm %2d.%dV",
									buf2[8 + g],
									buf2[4 + g] / 10,
									buf2[4 + g] % 10);
								if (3 != g && 3 > modi[g + 1] && 0 < lz[g +1]) {
									fprintf(fomlog, "${color0}, ");
								} else {
									fprintf(fomlog, " ");
								}
							}
						} else {
							/* Durchflussausgabe anstellen */
							i = 1;
						}
					}
					/* Durchfluss in fomlog schreiben */
					if (i) {
						fprintf(fomlog, "${color0}DFM:${color1}%3d l/h ", buf2[22]);
					}
					wprintw(links, "\nFOM-Display nach %d min ausschalten",
							dto);
					/* Sensoren ausgeben */
					fprintf(fomlog, "${color0}${font weather:size=11}x${font Impact:size=10}${color1}");
					for (g = 0; g < 6; g++) {
						if (1 == (g & 1)) {
							wattron(rechts, COLOR_PAIR(3));
						}
						wprintw(rechts, "Sensor%d Name: ", g + 1);
						for (i = 0; i < 20; i++) {
							ausEinsMachZwei(namen[g + 4][i], rechts);
						}
						wprintw(rechts, "\nTemp:%3d Solltemp: %2d Meldung: %2d "
								"Mobo: %2d Relais: %2d 12v: %2d\n",
								buf2[13 + g], solltemp[g], meld[g + 4],
								mobo[g + 4], relais[g + 4], aus12V[g + 4]);
						if (1 == (g & 1)) {
							wattroff(rechts, COLOR_PAIR(3));
						}
						/* Sensortemperaturen in fomlog schreiben */
						fprintf(fomlog, "%3d", buf2[13 + g]);
						if (5 != g) {
							fprintf(fomlog, " ");
						}
					}
					/* Durchfluss ausgeben */
					wprintw(rechts, "\nDurchfluss:\n%2d.%2d l/min %3d l/h "
							"bei %3d Pulsen/Liter\nMeldung: %d.%02d "
							"Mobo: %d.%02d Relais: %d.%02d 12v: %d.%02d\n",
							(unsigned char)buf2[21] / 100,
							(unsigned char)buf2[21] % 100, buf2[22] & 0xFF,
							pulse, (unsigned char)dfs[0] / 100,
							(unsigned char)dfs[0] % 100,
							(unsigned char)dfs[1] / 100,
							(unsigned char)dfs[1] % 100,
							(unsigned char)dfs[2] / 100,
							(unsigned char)dfs[2] % 100,
							(unsigned char)dfs[3] / 100,
							(unsigned char)dfs[3] % 100);
					wattron(rechts, COLOR_PAIR(3));
					wprintw(rechts, "\n12V: %2d.%dV 5V:%2d.%dV",
							buf2[19] / 10, buf2[19] % 10,
							buf2[20] / 10, buf2[20] % 10);
					wattroff(rechts, COLOR_PAIR(3));
					gebeAlarmmeldungenAus(buf2);
					wrefresh(links);
					wrefresh(rechts);
					pthread_mutex_unlock(&mutex3);
					/* Datei fom.log schliessen */
					fclose(fomlog);
					break;
				}
			}
			/* Eine Viertelsekunde pennen laenger sollte FOM nicht brauchen
			   Das Gepenne wird durch SIGIO Signal unterbrochen */
			usleep(250000);
			if (!datenDa) {
				break;
			}
		}
		zaehler = 0;
	}
	pthread_join(thread, NULL);
}

/* Hilfsfunktion Konfigurationsmodus */
static bool bearbeiteAntwort(const char *meldung, unsigned char luefter) {
	int eingabe = 0;
	int i = 0;
	bool rueck = FALSE;
	char buf[8] = {0};
	char buf2[8] = {0};
	const unsigned char se[5] = {0x55, 0x09, 0x05, 0x93, 0x00};
	unsigned char zaehler = 0;

	pthread_mutex_lock(&mutex3);
	wclear(unten);
	mvwprintw(unten, 0, COLS / 2 - 20, "%s %d ", meldung, luefter + 1);
	pthread_mutex_unlock(&mutex3);
	while (TRUE) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		pthread_mutex_lock(&mutex2);
		if (!lauf) {
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			pthread_mutex_unlock(&mutex2);
			break;
		}
		pthread_mutex_unlock(&mutex2);
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		
		if (datenDa) {
			sigprocmask(SIG_BLOCK,&mask,&orig_mask);
			eingabe = (int)read(fd, buf, 8);
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			/* Mal schauen was ankam */
			for (i = 0; i < eingabe; i++) {
				buf2[zaehler++] = buf[i];
			}
			/* Daten wurden verarbeitet also warten */
			datenDa = FALSE;
		}
		if (5 < zaehler) {
			if (0x55 == buf2[0] && 0x09 == buf2[1] &&
					0x06 == buf2[2] &&
					0x91 == (unsigned char)buf2[3] &&
					0x00 == buf2[4] && 0x00 == buf2[5]) {
				/* Einstellungen sichern */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				if (-1 == write(fd,se,5)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				pthread_mutex_lock(&mutex3);
				waddstr(unten, "wurde übernommen...");
				pthread_mutex_unlock(&mutex3);
				zaehler = 0;
			/* Antwort auf 0x55 0x09 0x05 0x93 0x00 */
			} else if (0x55 == buf2[0] && 0x09 == buf2[1] &&
					0x06 == buf2[2] &&
					0x93 == (unsigned char)buf2[3] &&
					0x00 == buf2[4] && 0x00 == buf2[5]) {
				pthread_mutex_lock(&mutex3);
				waddstr(unten, "und gesichert!");
				rueck = TRUE;
				wrefresh(unten);
				pthread_mutex_unlock(&mutex3);
				break;
			} else {
				pthread_mutex_lock(&mutex3);
				waddstr(unten, "konnte nicht übernommen werden.");
				/*wprintw(unten, "%x %x %x %x %x %x",
						  buf2[0], buf2[1], buf2[2],
						  buf2[3], buf2[4], buf2[5]);*/
				rueck = FALSE;
				wrefresh(unten);
				pthread_mutex_unlock(&mutex3);
				break;
			}
		}
		/* 250 Millisekunden pennen
		   Das Gepenne wird durch SIGIO Signal unterbrochen */
		usleep(250000);
		if (!datenDa) {
			gingNichtGut("Verbindung zum FOM wurde gestört!");
		}
	}
	return rueck;
}

/* Hilfsfunktion */
static void beendeModus(pthread_t thread) {
	pthread_mutex_lock(&mutex);
	lauf = FALSE;
	weiter = TRUE;
	pthread_mutex_unlock(&mutex);
	delwin(unten);
	unten = NULL;
	pthread_join(thread, NULL);
}

/* Hilfsfunktion Konfigurationsmodus: Gibt Konfigurationsmenue aus */
static void gebeKonfAus(char luefterMenueEintraege[40][27], unsigned char woBinIch) {
	WINDOW *auswahl = NULL;
	unsigned char i = 0;
	unsigned char j = 0;
	wmove(links, 0, 0);
	wmove(rechts, 0, 0);
	for (i = 0; i < 40; i++) {
		if (0 == luefterMenueEintraege[i][0]) {
			continue;
		}
		if ((i % 8) < 4) {
			auswahl = links;
		} else {
			auswahl = rechts;
		}
		if (i == woBinIch) {
			wattron(auswahl, COLOR_PAIR(2));
		}
		if (1 == i) {
			for (j = 0; j < 26; j++) {
				ausEinsMachZwei(luefterMenueEintraege[1][j], links);
			}
		} else if (6 == i) {
			for (j = 0; j < 26; j++) {
				ausEinsMachZwei(luefterMenueEintraege[6][j], rechts);
			}
		} else {
			if (i != woBinIch && (i == 20 || (27 < i && i < 32))) {
				wattron(rechts, COLOR_PAIR(3));
			}
			waddstr(auswahl, luefterMenueEintraege[i]);
			if (i != woBinIch && (i == 20 || (27 < i && i < 32))) {
				wattroff(rechts, COLOR_PAIR(3));
			}
		}
		if (i == woBinIch) {
			wattroff(auswahl, COLOR_PAIR(2));
		}
	}
	wrefresh(links);
	wrefresh(rechts);
}

/* Hilfsfunktion Konfigurationsmodus: Gibt Tooltips aus */
static void gebeTooltipsAus(unsigned char zeile, unsigned char woBinIch) {
	wmove(unten, zeile + 1, 0);
	switch (woBinIch) {
		case 0:
			waddstr(unten, "Wählt den Lüfterkanal aus.");
			break;
		case 1:
			waddstr(unten, "Ändert den Namen "
					"des ausgewählten Lüfterkanals.");
			break;
		case 4:
			waddstr(unten, "Wählt den Sensorkanal aus.");
			break;
		case 5:
			waddstr(unten, "Stellt die Solltemperatur des ausgewählten "
					"Sensorkanals ein.\nIm automatischen Lüftermodus "
					"und diesem zugewiesenen Sensorkanal versucht der "
					"FOM die eingestellte Temperatur zu halten.");
			break;
		case 6:
			waddstr(unten, "Ändert den Namen "
					"des ausgewählten Sensorkanals.");
			break;
		case 8:
			waddstr(unten, "Wenn die hier eingestellte "
					"Drehzahl unterschritten wird, "
					"wird über die Software eine Warnung ausgegeben.");
			break;
		case 9:
			waddstr(unten, "Wenn die hier eingestellte "
					"Drehzahl unterschritten wird, "
					"wird das Signal zum Mainboard unterbrochen.");
			break;
		case 10:
			waddstr(unten, "Wenn die hier eingestellte "
					"Drehzahl unterschritten wird, "
					"wird der Relaisausgang geschaltet.");
			break;
		case 11:
			waddstr(unten, "Wenn die hier eingestellte "
					"Drehzahl unterschritten wird, "
					"wird der 12 Volt Ausgang geschaltet.");
			break;
		case 12:
			waddstr(unten, "Wenn die hier eingestellte "
					"Temperatur unterschritten wird, "
					"wird über die Software eine Warnung ausgegeben.");
			break;
		case 13:
			waddstr(unten, "Wenn die hier eingestellte "
					"Temperatur unterschritten wird, "
					"wird das Signal zum Mainboard unterbrochen.");
			break;
		case 14:
			waddstr(unten, "Wenn die hier eingestellte "
					"Temperatur unterschritten wird, "
					"wird der Relaisausgang geschaltet.");
			break;
		case 15:
			waddstr(unten, "Wenn die hier eingestellte "
					"Temperatur unterschritten wird, "
					"wird der 12 Volt Ausgang geschaltet.");
			break;
		case 16:
			waddstr(unten, "Stellt ein wieviele Pulse der Lüfter "
					"pro Umdrehung ausgibt. Standardwert: 2.");
			break;
		case 17:
			waddstr(unten, "Stellt die Mindestspannung des Lüfters ein."
					);
			break;
		case 18:
			waddstr(unten, "Stellt die Maximalspannung des Lüfters ein."
					);
			break;
		case 19:
			waddstr(unten, "Stellt die Boostspannung des Lüfters ein.");
			break;
		case 20:
			waddstr(unten, "Stellt ein wieviele Pulse der "
					"Durchflussmesser pro Liter ausgibt.");
			break;
		case 24:
			waddstr(unten, "Stellt ein, ob dieser Lüfter im "
					"automatischen Modus abgeschaltet werden "
					"darf oder er die Mindestspannung halten soll.");
			break;
		case 25:
			waddstr(unten, "Stellt den Zeitabstand (in Sekunden) "
					"zwischen den einzelnen Regelschritten ein.");
			break;
		case 26:
			waddstr(unten, "Dieser Wert legt die Spannungserhöhung (-"
					"verringerung) der folgenden Regelschritte fest.");
			break;
		case 27:
			waddstr(unten, "Mit diesem Wert wird die Spannungserhöhung "
					"(-verringerung) des ersten Regelschrittes "
					"festgelegt.");
			break;
		case 28:
			waddstr(unten, "Wenn der hier eingestellte "
					"Durchfluss unterschritten wird, "
					"wird über die Software eine Warnung ausgegeben.");
			break;
		case 29:
			waddstr(unten, "Wenn der hier eingestellte "
					"Durchfluss unterschritten wird, "
					"wird das Signal zum Mainboard unterbrochen.");
			break;
		case 30:
			waddstr(unten, "Wenn der hier eingestellte "
					"Durchfluss unterschritten wird, "
					"wird der Relaisausgang geschaltet.");
			break;
		case 31:
			waddstr(unten, "Wenn der hier eingestellte "
					"Durchfluss unterschritten wird, "
					"wird der 12 Volt Ausgang geschaltet.");
			break;
		case 32:
			waddstr(unten, "Stellt die Spannung des Lüfters "
					"im manuellen Lüftermodus ein.");
			break;
		case 33:
			waddstr(unten, "Stellt ein mit welcher Sensortemperatur "
					"der Lüfter im automatischen Lüftermodus "
					"geregelt werden soll.");
			break;
		case 34:
			waddstr(unten, "Stellt den Lüftermodus ein.\n"
					"Manuell: Die Spannung des Lüfters kann über "
					"die Software oder den FOM eingestellt werden.\n"
					"Automatik: Die Spannung des Lüfters "
					"wird automatisch angepasst.\n"
					"Flow-O-Matic: Ein Durchflussmesser muss "
					"angeschlossen sein. Es wird ausgegeben, ob ein "
					"Durchfluss vorhanden ist oder nicht.\n"
					"Flow-Meter: Ein Durchflussmesser muss ange"
					"schlossen sein. Der Durchfluss wird ausgegeben.");
			break;
		case 35:
			waddstr(unten, "Schaltet den Lüfter an oder aus.");
			break;
		case 36:
			waddstr(unten, "Schaltet den Relaisausgang.");
			break;
		case 37:
			waddstr(unten, "Schaltet den 12 Volt Ausgang.");
			break;
	}
	wrefresh(unten);
}

/* Hilfsfunktion Konfigurationsmodus: Passt das Fenster an */
static void passeFenstiAn(unsigned char woBinIch) {
	sigprocmask(SIG_BLOCK,&mask,&orig_mask);
	pthread_mutex_lock(&mutex3);
	if (fensterNeu) {
		passeFensterAn();
		starteMenue();
		gebeTooltipsAus(56 / (COLS - 2), woBinIch);
	}
	pthread_mutex_unlock(&mutex3);
	sigprocmask(SIG_SETMASK,&orig_mask,NULL);
}

static void *frageAlarmAb(void *data) {
	ssize_t g = 0;
	FILE *fomlog = NULL; /* Filezeiger auf die Datei fom.log */
	char pfadname[128] = {0};
	char buf[256] = {0};
	char buf2[256] = {0};
	/* Anfragefolge fuer Daten vom Fom */
	const unsigned char ds[5] = {0x55,0x09,0x05,0x13,0x00};
	unsigned char i = 0;
	unsigned char zaehler = 0;

	/* pfadnamen zusammenbasteln */
	snprintf(pfadname, 128, "%s/fom.log", getenv("HOME"));

	/* Programm beenden mit SIGINT oder SIGTERM */
	while (TRUE) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		pthread_mutex_lock(&mutex);
		if (!lauf || weiter) {
			pthread_mutex_unlock(&mutex);
			return data;
		}
		pthread_mutex_unlock(&mutex);
		pthread_mutex_lock(&mutex4);
		if (-1 == write(fd,ds,5)) {
			gingNichtGut("write schlug fehl!");
		}
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		while (TRUE) {
			sigprocmask(SIG_BLOCK,&mask,&orig_mask);
			pthread_mutex_lock(&mutex2);
			if (!lauf) {
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				pthread_mutex_unlock(&mutex2);
				return data;
			}
			pthread_mutex_unlock(&mutex2);
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			if (datenDa) {
				/* Bytes lesen und ausgeben */
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				g = read(fd,buf,128);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Mal schauen was ankam */
				for (i = 0; i < g; i++) {
					buf2[zaehler++] = buf[i];
				}
				/* Daten wurden verarbeitet also warten */
				datenDa = FALSE;
				if (25 < zaehler) {
					pthread_mutex_unlock(&mutex4);
					/* fom.log Datei erstellen oder oeffnen */
					if ( NULL == (fomlog = fopen(pfadname, "w")) ) {
						perror("fopen fom.log");
						pthread_mutex_lock(&mutex2);
						lauf = FALSE;
						pthread_mutex_unlock(&mutex2);
						break;
					}
					/* Luefter ausgeben */
					fprintf(fomlog, "Lüfter: ");
					for (g = 0; g < 4; g++) {
						if (modi[g] < 3) {
							if (0 < lz[g]) {
								fprintf(fomlog, "${color1}%2d00 rpm %2d.%dV",
									buf2[8 + g],
									buf2[4 + g] / 10,
									buf2[4 + g] % 10);
								if (3 != g && 3 > modi[g + 1] && 0 < lz[g +1]) {
									fprintf(fomlog, "${color0}, ");
								} else {
									fprintf(fomlog, " ");
								}
							}
						} else {
							/* Durchflussausgabe anstellen */
							i = 1;
						}
					}
					/* Durchfluss in fomlog schreiben */
					if (i) {
						fprintf(fomlog, "${color0}DFM:${color1}%3d l/h ", buf2[22]);
					}
					/* Sensoren ausgeben */
					fprintf(fomlog, "${color0}${font weather:size=11}x${font Impact:size=10}${color1}");
					for (g = 0; g < 6; g++) {
						/* Sensortemperaturen in fomlog schreiben */
						fprintf(fomlog, "%3d", buf2[13 + g]);
						if (5 != g) {
							fprintf(fomlog, " ");
						}
					}
					sigprocmask(SIG_BLOCK,&mask,&orig_mask);
					pthread_mutex_lock(&mutex3);
					gebeAlarmmeldungenAus(buf2);
					wrefresh(links);
					pthread_mutex_unlock(&mutex3);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					/* Datei fom.log schliessen */
					fclose(fomlog);
					break;
				}
			}
			/* Eine Viertelsekunde pennen laenger sollte FOM nicht brauchen
			   Das Gepenne wird durch SIGIO Signal unterbrochen */
			usleep(250000);
			if (!datenDa) {
				pthread_mutex_unlock(&mutex4);
				break;
			}
		}
		zaehler = 0;
	}
	return data;
}

/* Konfigurationsmodus */
static void stelleFOMein() {
	int eingabe = 0;
	int tmp = 0;
	int zu = 0;
	pthread_t thread = 0;
	char luefterMenueEintraege[40][27] = {{0}};
	char name[21] = {0};
	static char rel[2] = {0}; /* Relais geschaltet oder nicht */
	unsigned char i = 0;
	char buf[8] = {0};
	char buf2[8] = {0};
	unsigned char is[6] = {0x55, 0x09, 0x06, 0x91, 0x11, 0x01};
	unsigned char aktuell = 99;
	unsigned char spalte = 0;
	char sendeNamen[26] = {0x55, 0x09, 0x1A, 0x50, 0x00};
	unsigned char zaehler = 0;
	unsigned char zeile = 0;
	unsigned char woBinIch = 0;
	unsigned char luefter = 0;
	unsigned char sensor = 0;
	const char *lm[] = {"Manuell     ", "Automatik   ",
			"Flow-O-Matic", "Flow-Meter  "};
	const char *laa[] = {"aus", "an "};

	/* Alarmausgeben Faden starten */
	pthread_create(&thread, NULL, frageAlarmAb, NULL);
	
	while (TRUE) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		pthread_mutex_lock(&mutex2);
		if (!lauf) {
			pthread_mutex_unlock(&mutex2);
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			break;
		}
		pthread_mutex_unlock(&mutex2);
		pthread_mutex_lock(&mutex3);
		if (fensterNeu) {
			passeFensterAn();
			starteMenue();
		}
		pthread_mutex_unlock(&mutex3);
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		/*mvwprintw(unten, 0, COLS / 2 - 10, "Umbrüche: %d", tmp);
		wrefresh(unten);*/
		if (2 > aktuell || 5 == aktuell || 99 == aktuell) {
			pthread_mutex_lock(&mutex3);
			/* Wieviele Zeilen benoetigt die Einstellungsausgabe? */
			zeile = 56 / (COLS - 2);
			/* Die Hilfsausgabe bereinigen */
			wmove(unten, zeile + 1, 0);
			for (eingabe = 0; eingabe < LINES - 9; eingabe++) {
				for(tmp = 0; tmp < COLS - 2; tmp++) {
					waddch(unten, ' ');
				}
			}
			/* Hilfsmenue ausgeben */
			gebeTooltipsAus(zeile, woBinIch);
			pthread_mutex_unlock(&mutex3);
			/* Tasteneingaben abfragen um durch das Menue zu navigieren */
			if (0 == aktuell) {
				eingabe = wgetch(links);
				zeile = woBinIch / 8;
				switch (eingabe) {
					case KEY_UP:
						if (10 == woBinIch || 11 == woBinIch || 6 == woBinIch
								|| 29 == woBinIch || 30 == woBinIch
								|| 31 == woBinIch) {
							woBinIch = (5 * 8 + woBinIch  - 2 * 8) % (5 * 8);
						} else if (15 == woBinIch) {
							woBinIch = 31;
						} else {
							woBinIch = (5 * 8 + woBinIch - 8) % (5 * 8);
						}
						break;
					case KEY_DOWN:
						if (30 == woBinIch || 34 == woBinIch || 35 == woBinIch
								|| 13 == woBinIch || 14 == woBinIch
								|| 15 == woBinIch) {
							woBinIch = (woBinIch + 2 * 8) % (5 * 8);
						} else if (31 == woBinIch) {
							woBinIch = 15;
						} else {
							woBinIch = (woBinIch + 8) % (5 * 8);
						}
						break;
					case KEY_LEFT:
						if (0 == woBinIch) {
							woBinIch = 6;
						} else if (4 == woBinIch) {
							woBinIch = 1;
						} else if (16 == woBinIch) {
							woBinIch = 20;
						} else if (32 == woBinIch) {
							woBinIch = 37;
						} else {
							woBinIch = (8 + woBinIch - 1) % 8 + zeile * 8;
						}
						break;
					case KEY_RIGHT:
						if (1 == woBinIch) {
							woBinIch = 4;
						} else if (6 == woBinIch) {
							woBinIch = 0;
						} else if (20 == woBinIch) {
							woBinIch = 16;
						} else if (37 == woBinIch) {
							woBinIch = 32;
						} else {
							woBinIch = (woBinIch + 1) % 8 + zeile * 8;
						}
						break;
					case 27:
						aktuell = 255;
						break;
					case (int)'\n':
						aktuell = woBinIch + 1;
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
			/* Modus Anfangsausgabe zuruecksetzen */
			} else if (99 == aktuell) {
				aktuell = 0;
			} else {
				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						/* Luefter auswaehlen */
						if (1 == aktuell) {
							luefter = (luefter + 1) % 4;
						/* Sensor auswaehlen */
						} else {
							sensor = (sensor + 1) % 6;
						}
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						/* Luefter auswaehlen */
						if (1 == aktuell) {
							luefter = (4 + luefter - 1) % 4;
						/* Sensor auswaehlen */
						} else {
							sensor = (6 + sensor - 1) % 6;
						}
						break;
					case 27:
					case (int)'\n':
						aktuell = 0;
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
			}
			/* Eintraege Konfigurationsmenue erstellen */
			snprintf(luefterMenueEintraege[0], 10, "Lüfter%d ", luefter + 1);
			snprintf(luefterMenueEintraege[1], 27, "Name: %s", namen[luefter]);
			snprintf(luefterMenueEintraege[4], 9, "Sensor%d ", sensor + 1);
			snprintf(luefterMenueEintraege[5], 20, "Solltemperatur: %2d ",
					solltemp[sensor]);
			snprintf(luefterMenueEintraege[6], 27, "Name: %s",
					namen[sensor + 4]);
			snprintf(luefterMenueEintraege[8], 15, "\nMeldung:%2d00 ",
					 meld[luefter]);
			snprintf(luefterMenueEintraege[9], 11, "Mobo:%2d00 ",
					 mobo[luefter]);
			snprintf(luefterMenueEintraege[10], 13, "Relais:%2d00 ",
					 relais[luefter]);
			snprintf(luefterMenueEintraege[11], 9, "12v:%2d00",
					 aus12V[luefter]);
			snprintf(luefterMenueEintraege[12], 14, "\nMeldung: %2d ",
					meld[sensor + 4]);
			snprintf(luefterMenueEintraege[13], 10, "Mobo: %2d ",
					mobo[sensor + 4]);
			snprintf(luefterMenueEintraege[14], 12, "Relais: %2d ",
					relais[sensor + 4]);
			snprintf(luefterMenueEintraege[15], 8, "12v: %2d",
					aus12V[sensor + 4]);
			snprintf(luefterMenueEintraege[16], 11, "\nPulse:%2d ",
					 lp[luefter]);
			snprintf(luefterMenueEintraege[17], 12, "Umin: %2d.%d ",
					min[luefter] / 10, min[luefter] % 10);
			snprintf(luefterMenueEintraege[18], 12, "Umax: %2d.%d ",
					max[luefter] / 10, max[luefter] % 10);
			snprintf(luefterMenueEintraege[19], 12, "Boost: %2d.%d",
					boost[luefter] / 10, boost[luefter] % 10);
			snprintf(luefterMenueEintraege[20], 16,
					"\nDFM: %3d Pulse", pulse);
			snprintf(luefterMenueEintraege[24], 15, "\nFan-Off: %s ",
					laa[(int)fo[luefter]]);
			snprintf(luefterMenueEintraege[25], 10, "Zeit: %2d ",
					 zeit[luefter]);
			snprintf(luefterMenueEintraege[26], 15, "Integral: %d.%d ",
					ig[luefter] / 10, ig[luefter] % 10);
			snprintf(luefterMenueEintraege[27], 18, "Proportional: %d.%d",
					prop[luefter] / 10, prop[luefter] % 10);
			snprintf(luefterMenueEintraege[28], 16,
					"\nMeldung: %d.%02d ", (unsigned char)dfs[0] / 100,
					 (unsigned char)dfs[0] % 100);
			snprintf(luefterMenueEintraege[29], 12,
					"Mobo: %d.%02d ", (unsigned char)dfs[1] / 100,
					 (unsigned char)dfs[1] % 100);
			snprintf(luefterMenueEintraege[30], 14,
					"Relais: %d.%02d ", (unsigned char)dfs[2] / 100,
					 (unsigned char)dfs[2] % 100);
			snprintf(luefterMenueEintraege[31], 10,
					"12v: %d.%02d", (unsigned char)dfs[3] / 100,
					 (unsigned char)dfs[3] % 100);
			snprintf(luefterMenueEintraege[32], 17, "\nSpannung: %2d.%d ",
					span[luefter] / 10, span[luefter] % 10);
			snprintf(luefterMenueEintraege[33], 11, "Sensor: %d ",
					sensoren[luefter]);
			strncpy(luefterMenueEintraege[34], lm[(int)modi[luefter]], 13);
			snprintf(luefterMenueEintraege[35], 14, " Lüfter: %s",
					laa[(int)lz[luefter]]);
			snprintf(luefterMenueEintraege[36], 14, "\nRelais: %s ",
					laa[(int)rel[0]]);
			snprintf(luefterMenueEintraege[37], 17, "12V-Ausgang: %s",
					laa[(int)rel[1]]);
		/* Navigation Obermenue */
		} else if (255 == aktuell) {
			if (frageTasteAb()) {
				delwin(unten);
				unten = NULL;
				pthread_mutex_lock(&mutex);
				weiter = TRUE;
				pthread_mutex_unlock(&mutex);
				break;
			}
		/* Namen der Luefter aendern */
		} else if (2 == aktuell) {
			snprintf(luefterMenueEintraege[1], 27,
				 "Name:                     ");
			pthread_mutex_lock(&mutex3);
			gebeKonfAus(luefterMenueEintraege, woBinIch);
			wattron(links, COLOR_PAIR(2));
			curs_set(2);
			zu = COLS / 2 - 2;
			pthread_mutex_unlock(&mutex3);
			for (i = 0; i < 21; i++) {
				zeile = (14 + i) / zu;
				spalte = (14 + i) % zu;
				eingabe = mvwgetch(links, zeile, spalte);
				switch (eingabe) {
					case KEY_UP:
					case KEY_DOWN:
					case KEY_LEFT:
					case KEY_RIGHT:
						i--;
						break;
					case 194:
					case 195:
						tmp = wgetch(links);
						pthread_mutex_lock(&mutex3);
						name[i] = ausZweiMachEins(eingabe,
								tmp, zeile, spalte, links);
						pthread_mutex_unlock(&mutex3);
						break;
					case KEY_BACKSPACE:
					case 127:
						if (0 != i) {
							i = i - 2;
						} else {
							i--;
						}
						pthread_mutex_lock(&mutex3);
						mvwaddch(links, zeile, spalte - 1, ' ');
						pthread_mutex_unlock(&mutex3);
						name[i - 1] = ' ';
						break;
					case 27:
						break;
					case KEY_RESIZE:
					case ERR:
						sigprocmask(SIG_BLOCK,&mask,&orig_mask);
						pthread_mutex_lock(&mutex3);
						if (fensterNeu) {
							passeFensterAn();
							starteMenue();
							for (tmp = (int) i; tmp < 20; tmp++) {
								name[tmp] = ' ';
							}
							name[20] = '\0';
							snprintf(luefterMenueEintraege[1], 27, "Name: %s", name);
							gebeKonfAus(luefterMenueEintraege, woBinIch);
							gebeTooltipsAus(56 / (COLS - 2), woBinIch);
							wattron(links, COLOR_PAIR(2));
							curs_set(2);
						}
						pthread_mutex_unlock(&mutex3);
						sigprocmask(SIG_SETMASK,&orig_mask,NULL);
						i--;
						continue;
					default:
						if ((int)'\n' == eingabe) {
							for (tmp = (int) i; tmp < 20; tmp++) {
								name[tmp] = ' ';
							}
							name[20] = '\n';
							break;
						}
						name[i] = (char) eingabe;
						pthread_mutex_lock(&mutex3);
						if (20 == i) {
							mvwaddstr(unten, 0, COLS / 2 - 14,
									  "21. Zeichen war kein enter!");
							wrefresh(unten);
							pthread_mutex_lock(&mutex3);
							break;
						}
						mvwaddch(links, zeile, spalte, eingabe);
						pthread_mutex_unlock(&mutex3);
				}
				if (27 == eingabe || (int) '\n' == eingabe) {
					break;
				}
				pthread_mutex_lock(&mutex3);
				wrefresh(links);
				pthread_mutex_unlock(&mutex3);
			}
			if ('\n' == name[20] && 27 != eingabe) {
				name[20] = '\0';
				sendeNamen[5] = 0x82 + luefter;
				strncpy(&sendeNamen[6], name, 20);
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd, sendeNamen, 26)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				while (TRUE) {
					sigprocmask(SIG_BLOCK,&mask,&orig_mask);
					pthread_mutex_lock(&mutex2);
					if (!lauf) {
						pthread_mutex_unlock(&mutex2);
						sigprocmask(SIG_SETMASK,&orig_mask,NULL);
						break;
					}
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					if (datenDa) {
						sigprocmask(SIG_BLOCK,&mask,&orig_mask);
						eingabe = read(fd, buf, 8);
						sigprocmask(SIG_SETMASK,&orig_mask,NULL);
						for (i = 0; i < eingabe; i++) {
							buf2[zaehler++] = buf[i];
						}
						datenDa = FALSE;
					}
					if (4 < zaehler) {
						pthread_mutex_unlock(&mutex4);
						pthread_mutex_lock(&mutex3);
						mvwprintw(unten, 0, COLS / 2 - 10,
								"Name %d Zeichen %d ", luefter + 1, i);
						if (0x55 == buf2[0] && 0x09 == buf2[1] &&
							0x06 == buf2[2] && 0x50 == buf2[3] &&
							0x00 == buf2[4] && 0x82 + luefter ==
							(unsigned char)buf2[5]) {
							waddstr(unten, "wurde geschrieben.");
						} else {
							waddstr(unten,
									"konnte nicht geschrieben werden.");
							/*wprintw(unten, "%x %x %x %x %x %x",
										  buf2[0], buf2[1], buf2[2],
										  buf2[3], buf2[4], buf2[5]);*/
						}
						wrefresh(unten);
						pthread_mutex_unlock(&mutex3);
						zaehler = 0;
						break;
					}
					usleep(250000);
					if (!datenDa) {
						pthread_mutex_unlock(&mutex4);
						gingNichtGut("Verbindung zum FOM wurde gestört!");
					}
				}
				strncpy(namen[luefter], name, 20);
				namen[luefter][20] = '\0';
				snprintf(luefterMenueEintraege[1], 27, "Name: %s", namen[luefter]);
			}
			aktuell = 0;
			pthread_mutex_lock(&mutex3);
			wattroff(links, COLOR_PAIR(2));
			curs_set(0);
			pthread_mutex_unlock(&mutex3);
		/* Solltemperatur Sensor einstellen */
		} else if (6 == aktuell) {
			tmp = solltemp[sensor];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp - 10 + 1) % 90 + 10;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (90 + tmp - 10 - 1) % 90 + 10;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[5], 20, "Solltemperatur: %2d ",
					tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Solltemperatur uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = sensor * 5 + 56;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Solltemperatur Sensor", sensor)) {
					solltemp[sensor] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Namen der Sensoren aendern */
		} else if (7 == aktuell) {
			snprintf(luefterMenueEintraege[6], 27,
				 "Name:                     ");
			pthread_mutex_lock(&mutex3);
			gebeKonfAus(luefterMenueEintraege, woBinIch);
			wattron(rechts, COLOR_PAIR(2));
			curs_set(2);
			zu = COLS / 2 - 2;
			pthread_mutex_unlock(&mutex3);
			for (i = 0; i < 21; i++) {
				zeile = (33 + i) / zu;
				spalte = (33 + i) % zu;
				eingabe = mvwgetch(rechts, zeile, spalte);
				switch (eingabe) {
					case KEY_UP:
					case KEY_DOWN:
					case KEY_LEFT:
					case KEY_RIGHT:
						i--;
						break;
					case 194:
					case 195:
						tmp = wgetch(rechts);
						pthread_mutex_lock(&mutex3);
						name[i] = ausZweiMachEins(eingabe,
								tmp, zeile, spalte, rechts);
						pthread_mutex_unlock(&mutex3);
						break;
					case KEY_BACKSPACE:
					case 127:
						if (0 != i) {
							i = i - 2;
						} else {
							i--;
						}
						pthread_mutex_lock(&mutex3);
						mvwaddch(rechts, zeile, spalte - 1, ' ');
						pthread_mutex_unlock(&mutex3);
						name[i - 1] = ' ';
						break;
					case 27:
						break;
					case KEY_RESIZE:
					case ERR:
						sigprocmask(SIG_BLOCK,&mask,&orig_mask);
						pthread_mutex_lock(&mutex3);
						if (fensterNeu) {
							passeFensterAn();
							starteMenue();
							for (tmp = (int) i; tmp < 20; tmp++) {
								name[tmp] = ' ';
							}
							name[20] = '\0';
							snprintf(luefterMenueEintraege[6], 27, "Name: %s", name);
							gebeKonfAus(luefterMenueEintraege, woBinIch);
							gebeTooltipsAus(56 / (COLS - 2), woBinIch);
							wattron(rechts, COLOR_PAIR(2));
							curs_set(2);
						}
						pthread_mutex_unlock(&mutex3);
						sigprocmask(SIG_SETMASK,&orig_mask,NULL);
						i--;
						continue;
					default:
						if ((int)'\n' == eingabe) {
							for (tmp = (int) i; tmp < 20; tmp++) {
								name[tmp] = ' ';
							}
							name[20] = '\n';
							break;
						}
						name[i] = (char) eingabe;
						pthread_mutex_lock(&mutex3);
						if (20 == i) {
							mvwaddstr(unten, 0, COLS / 2 - 14,
									  "21. Zeichen war kein enter!");
							wrefresh(unten);
							pthread_mutex_unlock(&mutex3);
							break;
						}
						mvwaddch(rechts, zeile, spalte, eingabe);
						pthread_mutex_unlock(&mutex3);
				}
				if (27 == eingabe || (int) '\n' == eingabe) {
					break;
				}
				pthread_mutex_lock(&mutex3);
				wrefresh(rechts);
				pthread_mutex_unlock(&mutex3);
			}
			if ('\n' == name[20] && 27 != eingabe) {
				name[20] = '\0';
				sendeNamen[5] = 0x86 + sensor;
				strncpy(&sendeNamen[6], name, 20);
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd, sendeNamen, 26)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				while (TRUE) {
					pthread_mutex_lock(&mutex2);
					sigprocmask(SIG_BLOCK,&mask,&orig_mask);
					if (!lauf) {
						sigprocmask(SIG_SETMASK,&orig_mask,NULL);
						pthread_mutex_unlock(&mutex2);
						break;
					}
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					if (datenDa) {
						sigprocmask(SIG_BLOCK,&mask,&orig_mask);
						eingabe = read(fd, buf, 8);
						sigprocmask(SIG_SETMASK,&orig_mask,NULL);
						for (i = 0; i < eingabe; i++) {
							buf2[zaehler++] = buf[i];
						}
						datenDa = FALSE;
					}
					if (4 < zaehler) {
						pthread_mutex_unlock(&mutex4);
						pthread_mutex_lock(&mutex3);
						mvwprintw(unten, 0, COLS / 2 - 10,
								"Name %d Zeichen %d ", luefter + 1, i);
						if (0x55 == buf2[0] && 0x09 == buf2[1] &&
							0x06 == buf2[2] && 0x50 == buf2[3] &&
							0x00 == buf2[4] && 0x86 + sensor ==
							(unsigned char)buf2[5]) {
							waddstr(unten, "wurde geschrieben.");
						} else {
							waddstr(unten,
									"konnte nicht geschrieben werden.");
							/*wprintw(rechts, "%x %x %x %x %x %x",
										  buf2[0], buf2[1], buf2[2],
										  buf2[3], buf2[4], buf2[5]);*/
						}
						wrefresh(unten);
						pthread_mutex_unlock(&mutex3);
						zaehler = 0;
						break;
					}
					usleep(250000);
					if (!datenDa) {
						pthread_mutex_unlock(&mutex4);
						gingNichtGut("Verbindung zum FOM wurde gestört!");
					}
				}
				strncpy(namen[4 + sensor], name, 20);
				namen[4 + sensor][20] = '\0';
				snprintf(luefterMenueEintraege[6], 27, "Name: %s",
					namen[sensor + 4]);
			}
			aktuell = 0;
			pthread_mutex_lock(&mutex3);
			wattroff(rechts, COLOR_PAIR(2));
			curs_set(0);
			pthread_mutex_unlock(&mutex3);
		/* Alarm Meldung Luefter umstellen */
		} else if (9 == aktuell) {
			tmp = meld[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[8], 15, "\nMeldung:%2d00 ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Meldung Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 1;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Meldung Lüfter", luefter)) {
					meld[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Mainboard Alarm des Luefters umstellen */
		} else if (10 == aktuell) {
			tmp = mobo[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[9], 11, "Mobo:%2d00 ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Mainboard Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 2;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Mainboard Lüfter", luefter)) {
					mobo[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Relaisausgang Alarm Luefter umstellen */
		} else if (11 == aktuell) {
			tmp = relais[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[10], 13, "Relais:%2d00 ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Relaisausgang Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 3;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Relais Lüfter", luefter)) {
					relais[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* 12 Volt Ausgang Luefter umstellen */
		} else if (12 == aktuell) {
			tmp = aus12V[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[11], 9, "12v:%2d00", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen 12V Ausgang Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 4;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm 12 Volt Lüfter", luefter)) {
					aus12V[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Alarm Meldung Sensor umstellen */
		} else if (13 == aktuell) {
			tmp = meld[sensor + 4];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[12], 14, "\nMeldung: %2d ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Meldung Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = sensor * 5 + 57;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Meldung Sensor", sensor)) {
					meld[sensor + 4] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Mainboard Alarm Sensor umstellen */
		} else if (14 == aktuell) {
			tmp = mobo[sensor + 4];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[13], 10, "Mobo: %2d ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Mainboard Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = sensor * 5 + 58;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Mainboard Sensor", sensor)) {
					mobo[sensor + 4] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Relaisausgang Alarm Sensor umstellen */
		} else if (15 == aktuell) {
			tmp = relais[sensor + 4];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[14], 12, "Relais: %2d ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Relaisausgang Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = sensor * 5 + 59;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Relais Sensor", sensor)) {
					relais[sensor + 4] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* 12 Volt Ausgang Sensor umstellen */
		} else if (16 == aktuell) {
			tmp = aus12V[sensor + 4];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 100;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (100 + tmp - 1) % 100;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[15], 8, "12v: %2d", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen 12V Ausgang Alarm uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = sensor * 5 + 60;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm 12 Volt Sensor", sensor)) {
					aus12V[sensor + 4] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Pulse des Luefters umstellen */
		} else if (17 == aktuell) {
			tmp = lp[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp - 2 + 1) % 31 + 2;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (31 + tmp - 2 - 1) % 31 + 2;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[16], 11, "\nPulse:%2d ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Pulseinstellung des Luefters uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 6;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Impulseinstellung Lüfter", luefter)) {
					lp[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Minimalspannung des Luefters umstellen */
		} else if (18 == aktuell) {
			tmp = min[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 121;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (121 + tmp - 1) % 121;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[17], 12, "Umin: %2d.%d ",
						tmp / 10, tmp % 10);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Minimalspannungseinstellung uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 7;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Minimalspannung Lüfter", luefter)) {
					min[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Maximalspannung des Luefters umstellen */
		} else if (19 == aktuell) {
			tmp = max[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 121;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (121 + tmp - 1) % 121;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[18], 12, "Umax: %2d.%d ",
						tmp / 10, tmp % 10);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Maximalspannungseinstellung uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 8;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Maximalspannung Lüfter", luefter)) {
					max[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Boostspannung des Luefters umstellen */
		} else if (20 == aktuell) {
			tmp = boost[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 121;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (121 + tmp - 1) % 121;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[19], 12, "Boost: %2d.%d",
						tmp / 10, tmp % 10);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Boostspannung uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 9;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Boostspannung Lüfter", luefter)) {
					boost[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Pulse DFM umstellen */
		} else if (21 == aktuell) {
			tmp = pulse;
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 4336;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (4336 + tmp - 1) % 4336;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[20], 16,
						"\nDFM: %3d Pulse", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Pulse DFM uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = 91;
				is[5] = tmp & 0xFF;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Pulse niederwertig DFM", 1)) {
					is[4] = 92;
					is[5] = tmp >> 8;
					/* irgendwas beschreiben */
					sigprocmask(SIG_BLOCK,&mask,&orig_mask);
					pthread_mutex_lock(&mutex4);
					if (-1 == write(fd,is,6)) {
						gingNichtGut("write schlug fehl!");
					}
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					if (bearbeiteAntwort("Pulse höherwertig DFM", 1)) {
						pulse = tmp;
					}
					pthread_mutex_unlock(&mutex4);
				}
			}
			aktuell = 0;
		/* Fan-Off-Einstellung umstellen */
		} else if (25 == aktuell) {
			tmp = fo[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 2;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (2 + tmp - 1) % 2;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[24], 15, "\nFan-Off: %s ",
						laa[tmp]);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Fan-Off-Einstellung uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 10;
				is[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Fan-Off Einstellung Lüfter", luefter)) {
					fo[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Regler Zeit umstellen */
		} else if (26 == aktuell) {
			tmp = zeit[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp - 1 + 1) % 50 + 1;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (50 + tmp - 1 - 1) % 50 + 1;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[25], 10, "Zeit: %2d ", tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Regler Zeit uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 11;
				is[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Regler Zeit Lüfter", luefter)) {
					zeit[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Integral Einstellung umstellen */
		} else if (27 == aktuell) {
			tmp = ig[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 61;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (61 + tmp - 1) % 61;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[26], 15, "Integral: %d.%d ",
						tmp / 10, tmp % 10);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Integral Einstellung uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 12;
				is[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Integral Einstellung Lüfter", luefter)) {
					ig[luefter] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Proportional Einstellung umstellen */
		} else if (28 == aktuell) {
			tmp = prop[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 61;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (61 + tmp - 1) % 61;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[27], 18, "Proportional: %d.%d",
						tmp / 10, tmp % 10);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Proportional Einstellung uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 13;
				is[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Meldung Lüfter", luefter)) {
					prop[luefter] = (char)tmp;
			 	}
			 	pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Alarm Meldung DFM umstellen */
		} else if (29 == aktuell) {
			tmp = dfs[0];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 251;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (251 + tmp - 1) % 251;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[28], 16,
					"\nMeldung: %d.%02d ", tmp / 100, tmp % 100);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Meldung Alarm DFM uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = 86;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Meldung DFM", 1)) {
					dfs[0] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Alarm Mobo DFM umstellen */
		} else if (30 == aktuell) {
			tmp = dfs[1];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 251;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (251 + tmp - 1) % 251;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[29], 12,
						"Mobo: %d.%02d ", tmp / 100, tmp % 100);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Mobo Alarm DFM uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = 87;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Mobo DFM", 1)) {
					dfs[1] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Alarm Relais DFM umstellen */
		} else if (31 == aktuell) {
			tmp = dfs[2];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 251;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (251 + tmp - 1) % 251;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[30], 14,
						"Relais: %d.%02d ", tmp / 100, tmp % 100);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Relais Alarm DFM uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = 88;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm Relais DFM", 1)) {
					dfs[2] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Alarm 12V Ausgang DFM umstellen */
		} else if (32 == aktuell) {
			tmp = dfs[3];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 251;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (251 + tmp - 1) % 251;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[31], 10,
						"12v: %d.%02d", tmp / 100, tmp % 100);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen 12V Alarm DFM uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = 89;
				is[5] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Alarm 12 Volt DFM", 1)) {
					dfs[3] = (char)tmp;
				}
				pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Spannung umstellen */
		} else if (33 == aktuell) {
			tmp = span[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 121;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (121 + tmp - 1) % 121;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[32], 17, "\nSpannung: %2d.%d ",
						tmp / 10, tmp % 10);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Spannung uebernehmen */
			if ((int)'\n' == eingabe) {
				char lsa[6] = {0x55, 0x09, 0x06, 0x20, 0x00, 0x00};
				lsa[4] = luefter;
				lsa[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,lsa,6)) {
					gingNichtGut("write schlug fehl!");
				}
				pthread_mutex_unlock(&mutex4);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Der FOM schickt auf diese Anfrage keine Antwort */
				span[luefter] = (char)tmp;
			}
			aktuell = 0;
		/* Regel-Sensor umstellen */
		} else if (34 == aktuell) {
			tmp = sensoren[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp - 1 + 1) % 6 + 1;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (6 + tmp - 1 - 1) % 6 + 1;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[33], 11, "Sensor: %d ",
						tmp);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neue Regel-Sensor uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 5;
				is[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Regel-Sensor Lüfter", luefter)) {
					sensoren[luefter] = (char)tmp;
			 	}
			 	pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Lueftermodus umstellen */
		} else if (35 == aktuell) {
			tmp = modi[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 4;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (4 + tmp - 1) % 4;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				strncpy(luefterMenueEintraege[34], lm[tmp], 13);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Neuen Lueftermodus uebernehmen */
			if ((int)'\n' == eingabe) {
				is[4] = luefter * 14 + 0;
				is[5] = (char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,is,6)) {
					gingNichtGut("write schlug fehl!");
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (bearbeiteAntwort("Lüftermodus Lüfter", luefter)) {
					modi[luefter] = (char)tmp;
			 	}
			 	pthread_mutex_unlock(&mutex4);
			}
			aktuell = 0;
		/* Luefter aus oder anschalten */
		} else if (36 == aktuell) {
			tmp = lz[luefter];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(links);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 2;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (2 + tmp - 1) % 2;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[35], 14, " Lüfter: %s",
						laa[tmp]);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Luefter an oder ausstellen */
			if ((int)'\n' == eingabe) {
				char saa[5] = {0x55, 0x09, 0x05, 0x21, 0x00};
				saa[3] = 0x21 + tmp;
				saa[4] = luefter;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,saa,5)) {
					gingNichtGut("write schlug fehl!");
				}
				pthread_mutex_unlock(&mutex4);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Der FOM antwortet auf diese Anfrage nicht */
				lz[luefter] = (char)tmp;
			}
			aktuell = 0;
		/* Relais Ausgang umstellen */
		} else if (37 == aktuell) {
			char relae[5] = {0x55, 0x09, 0x05, 0x24, 0x00};
			tmp = rel[0];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 2;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (2 + tmp - 1) % 2;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[36], 14, "\nRelais: %s ",
						laa[tmp]);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* Relais umschalten */
			if ((int)'\n' == eingabe) {
				is[4] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,relae,5)) {
					gingNichtGut("write schlug fehl!");
				}
				pthread_mutex_unlock(&mutex4);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Der FOM schickt auf diese Anfrage keine Antwort */
				rel[0] = (char)tmp;
			}
			aktuell = 0;
		/* 12V Ausgang umstellen */
		} else if (38 == aktuell) {
			char a12v[5] = {0x55, 0x09, 0x05, 0x23, 0x00};
			tmp = rel[1];
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					break;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
 				eingabe = wgetch(rechts);
				switch (eingabe) {
					case KEY_UP:
					case KEY_RIGHT:
						tmp = (tmp + 1) % 2;
						break;
					case KEY_DOWN:
					case KEY_LEFT:
						tmp = (2 + tmp - 1) % 2;
						break;
					case KEY_RESIZE:
					case ERR:
						passeFenstiAn(woBinIch);
						break;
					case (int)'q':
						beendeModus(thread);
						return;
				}
				if ((int)'\n' == eingabe || 27 == eingabe) {
					break;
				}
				snprintf(luefterMenueEintraege[37], 17, "12V-Ausgang: %s",
						laa[tmp]);
				pthread_mutex_lock(&mutex3);
				gebeKonfAus(luefterMenueEintraege, woBinIch);
				pthread_mutex_unlock(&mutex3);
			}
			/* 12V Ausgang umschalten */
			if ((int)'\n' == eingabe) {
				is[4] = (unsigned char)tmp;
				/* irgendwas beschreiben */
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex4);
				if (-1 == write(fd,a12v,5)) {
					gingNichtGut("write schlug fehl!");
				}
				pthread_mutex_unlock(&mutex4);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				/* Der FOM schickt auf diese Anfrage keine Antwort */
				rel[1] = (char)tmp;
			}
			aktuell = 0;
		/* Eigentlich unerreichbarer Code */
		} else {
			gingNichtGut("Unbekannter Zustand!");
		}
		gebeKonfAus(luefterMenueEintraege, woBinIch);
	}
	/* Warten bis der Alarmausgabefaden fertig ist */
	pthread_join(thread, NULL);
}

/* Modus: Texttool
   Liest Sprachdatei ein */
static void starteTexttool() {
	int i = 0;
	int j = 0;
	int g = 0;
	int tmp = 0;
	int zeichen = 0;
	int zaehler = 0;
	int zaehler2 = 0;
	pthread_t thread = 0;
	FILE *sprach = NULL;
	bool tab = FALSE;
	char buf[128] = {0};
	char buf2[128] = {0};
	/* Speicherort fuer in den EEPROM zu schreibende Daten */
	char zeile[20] = {0};
	/* Bytefolge zum EEPROM schreiben */
	char eB[26] = {0x55,0x09,0x1A,0x50,0x00};

	sigemptyset(&mask2);
	sigaddset(&mask2, SIGIO);

	/* Alarmausgeben Faden starten */
	pthread_create(&thread, NULL, frageAlarmAb, NULL);

	while (TRUE) {
		sigprocmask(SIG_BLOCK,&mask,&orig_mask);
		pthread_mutex_lock(&mutex2);
		if (!lauf) {
			pthread_mutex_unlock(&mutex2);
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			break;
		}
 		pthread_mutex_unlock(&mutex2);
		pthread_mutex_lock(&mutex3);
		if (fensterNeu) {
			passeFensterAn();
			starteMenue();
			/* Alle Zeichen aus der Eingabe loeschen */
			/*nodelay(links, TRUE);
			while (ERR != wgetch(links));
			nodelay(links, FALSE);*/
		}
		wclear(links);
		mvwaddstr(links, 0, (COLS - 2) / 2 - 15,
				  "Pfad zur Sprachdatei angeben:");
		/* Cursor und Text auf andere Farbe setzen */
		wattron(links, COLOR_PAIR(2));
		/* Cursor sehr sichtbar setzen */
		curs_set(2);
		if ( 0 != zaehler2 && 194 != (unsigned char)buf[zaehler2] &&
			195 != (unsigned char)buf[zaehler2] ) {
			mvwaddstr(links, 1, COLS / 2 - 16, buf);
		}
		j = (COLS - 2) / 2 - 15;
		pthread_mutex_unlock(&mutex3);
		sigprocmask(SIG_SETMASK,&orig_mask,NULL);
		for (i = zaehler2; i < 128; i++) {
			sigprocmask(SIG_BLOCK,&mask2,&orig_mask2);
			g = mvwgetch(links, 1, j + zaehler);
			sigprocmask(SIG_SETMASK,&orig_mask2,NULL);
			switch (g) {
				case KEY_UP:
				case KEY_DOWN:
				case KEY_LEFT:
				case KEY_RIGHT:
					i--;
					break;
				case KEY_BACKSPACE:
				case 127:
					if (0 != i) {
						if ( 194 == (unsigned char)buf[i - 2] ||
							195 == (unsigned char)buf[i - 2]) {
							buf[i - 2] = '\0';
							i = i - 3;
						} else {
							i = i - 2;
						}
						buf[i + 1] = '\0';
						pthread_mutex_lock(&mutex3);
						mvwaddch(links, 1, j + zaehler - 1, ' ');
						pthread_mutex_unlock(&mutex3);
						zaehler--;
					} else {
						i--;
					}
					break;
				case '\n':
				case 27:
				case KEY_RESIZE:
				case ERR:
					break;
				default:
					buf[i] = (unsigned char) g;
					buf[i + 1] = '\0';
					if ( 194 != (unsigned char)buf[i] &&
						195 != (unsigned char)buf[i]) {
						pthread_mutex_lock(&mutex3);
						mvwaddstr(links, 1, j, buf);
						pthread_mutex_unlock(&mutex3);
						zaehler++;
					}
			}
			if (27 == g || '\n' == g || KEY_RESIZE == g || ERR == g) {
				break;
			}
			pthread_mutex_lock(&mutex3);
			wrefresh(links);
			pthread_mutex_unlock(&mutex3);
		}
		pthread_mutex_lock(&mutex3);
		wattroff(links, COLOR_PAIR(2));
		pthread_mutex_unlock(&mutex3);
		if (KEY_RESIZE == g || ERR == g) {
			zaehler2 = i;
			continue;
		}
		if (128 == i) {
			buf[127] = '\0';
		} else {
			buf[i] = '\0';
		}
		zaehler = 0;
		zaehler2 = 0;
		
		/* Curser wieder unsichtbar setzen */
		pthread_mutex_lock(&mutex3);
		curs_set(0);
		pthread_mutex_unlock(&mutex3);
		if (27 == g) {
			while (!frageTasteAb()) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex3);
				if (fensterNeu) {
					passeFensterAn();
					starteMenue();
				}
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				mvwaddstr(links, 0, (COLS - 2) / 2 - 15,
						"Pfad zur Sprachdatei angeben:");
				wrefresh(links);
				pthread_mutex_unlock(&mutex3);
			};
			pthread_mutex_lock(&mutex);
			weiter = TRUE;
			pthread_mutex_unlock(&mutex);
			break;
		} else if (NULL == (sprach = fopen(buf,"r"))) {
			pthread_mutex_lock(&mutex3);
			mvwprintw(unten, 0, (COLS - 2) / 2 - 17,
					"Datei konnte nicht gefunden werden!");
			wrefresh(unten);
			pthread_mutex_unlock(&mutex3);
			continue;
		}
		/* Waehrend des Schreibvorgangs auf den EEPROM
		 * die Alarmabfrage stoppen, da ansonsten der Schreibvorgang
		 * unnoetig in die Laenge gezogen wird                        */
 		pthread_mutex_lock(&mutex4);
		for (j = 0; j < 256; j++) {
			/* Zeilen vom Filedescriptor einlesen */
			for (i = 0; i < 24; i++) {
				zeichen = fgetc(sprach);
				if (EOF == zeichen) {
					pthread_mutex_lock(&mutex3);
					mvwaddstr(unten, 0, (COLS - 2) / 2 - 10,
							"Dateiende erreicht!");
					wrefresh(unten);
					pthread_mutex_unlock(&mutex3);
					pthread_mutex_lock(&mutex);
					weiter = TRUE;
					pthread_mutex_unlock(&mutex);
					break;
				} else if ('\t' == zeichen) {
					tab = TRUE;
				} else if ('\n' == zeichen) {
					if (0 < zaehler2) {
						zaehler2++;
					}
					break;
					/* UTF-8 Kontroll Zeichen abfangen */
				} else if (194 == zeichen || 195 == zeichen) {
					tmp = fgetc(sprach);
					zeile[zaehler2++] = ausZweiMachEins(zeichen,
							tmp, 3, (COLS - 2) / 2 - 10, links);
				} else {
					if (tab && '\r' != zeichen) {
						zeile[zaehler2++] = (unsigned char)zeichen;
					}
					if (20 == zaehler2) {
						/* Den Rest der Zeile ignorieren */
						while ('\n' != fgetc(sprach));
						break;
					}
				}
			}
			tab = FALSE;
			
			if (0 == zaehler2) {
				/* Die Zeile war leer! Zur naechsten Speicherstelle springen */
				zaehler2 = 0;
				continue;
			}
			
			/* Speicheradresse angeben an die gesendet werden soll */
			eB[5] = j & 0xFF;
			/* zu schreibende Zeichen in zu sendende Bytefolge einfuegen */
			strncpy(&eB[6], zeile, 20);
			/* externen EEPROM beschreiben */
			sigprocmask(SIG_BLOCK,&mask,&orig_mask);
			if (-1 == write(fd,eB,26)) {
				pthread_mutex_lock(&mutex2);
				lauf = FALSE;
				pthread_mutex_unlock(&mutex2);
				pthread_join(thread, NULL);
				gingNichtGut("write schlug fehl!");
			}
			sigprocmask(SIG_SETMASK,&orig_mask,NULL);
			/* warten bis FOM den externen EEPROM beschrieben hat */
			while (TRUE) {
				sigprocmask(SIG_BLOCK,&mask,&orig_mask);
				pthread_mutex_lock(&mutex2);
				if (!lauf) {
					pthread_mutex_unlock(&mutex2);
					sigprocmask(SIG_SETMASK,&orig_mask,NULL);
					pthread_join(thread, NULL);
					return;
				}
				pthread_mutex_unlock(&mutex2);
				sigprocmask(SIG_SETMASK,&orig_mask,NULL);
				if (datenDa) {
					g = (int)read(fd,buf,128);
					/* Mal schauen was ankam */
					for (i = 0; i < g; i++) {
						buf2[zaehler++] = buf[i];
					}
					/* Daten wurden verarbeitet also warten */
					datenDa = FALSE;
				}
				if (4 < zaehler) {
					pthread_mutex_lock(&mutex3);
					mvwprintw(unten, 0, (COLS - 2) / 2 - 15,
							"Zeile %d ", j);
					if (0x55 == buf2[0] &&
							0x09 == buf2[1] && 0x06 == buf2[2] &&
							0x50 == buf2[3] && 0x00 == buf2[4] &&
							j == (unsigned char)buf2[5]) {
						waddstr(unten,"wurde geschrieben!\n");
					} else {
						waddstr(unten,"konnte nicht geschrieben werden!\n");
					}
					wrefresh(unten);
					pthread_mutex_unlock(&mutex3);
					break;
				}
				/* Ein wenig pennen */
				usleep(250000);
				if (!datenDa) {
					break;
				}
			}
			/* zaehler zuruecksetzen */
			zaehler = 0;
			zaehler2 = 0;
		}
		/* Schreibvorgang auf den EEPROM ist abgeschlossen */
		pthread_mutex_unlock(&mutex4);
		if (NULL != sprach) {
			fclose(sprach);
		}

	} /* Ende der while Schleife */
	/* Faden wieder einfangen */
	pthread_join(thread, NULL);
}

int main(void) {
	unsigned char i = 0;

	/* Auf Default Zeichensetzung stellen */
	if (!setlocale(LC_ALL, "")) {
		fprintf(stderr, "Elende Lokaleinstellungen.\n");
		return EXIT_FAILURE;
	}

	/* Schauen ob utf8 an ist */
	if (strcmp(nl_langinfo(CODESET), "UTF-8")) {
		fprintf(stderr, "Dieses Programm braucht UTF-8!");
		return EXIT_FAILURE;
	}

	/* Signalmaske zusammenbasteln */
	sigemptyset(&mask);
	sigaddset(&mask, SIGIO);
	sigaddset(&mask, SIGINT);
	sigaddset(&mask, SIGWINCH);

	/* Versuch mit dem FOM Verbindung aufzunehmen */
	oeffnePort();

	/* Verbindung pruefen */
	gebeIDaus();

	/* Mutex initialisieren */
	pthread_mutex_init(&mutex, NULL);
	pthread_mutex_init(&mutex2, NULL);
	pthread_mutex_init(&mutex3, NULL);
	pthread_mutex_init(&mutex4, NULL);

	/* Konfigurationsdaten abfragen */
	frageCfgAb();

	/* Namen der Luefter und Sensoren abfragen */
	for (i = 0; i < 10; i++) {
		holeNamen(i);
	}

	while (lauf) {
		fensterNeu = TRUE;
		switch (aktiv) {
			case 0:
				/* Monitordatenmodus anzeigen */
				gebeUebersichtAus();
				break;
			case 1:
				/* Konfigurationsmodus anzeigen */
				stelleFOMein();
				break;
			case 2:
				/* Textdisplaymodus anzeigen */
				starteTexttool();
				break;
		}
		weiter = FALSE;
	}

	/* Porteinstellungen zurueckschreiben */
	tcsetattr(fd,TCSANOW,&alt);

	/* Serielle Schnittstelle wieder schliessen */
	close(fd);

	/* Fenster schliessen */
	if (NULL != links) {
		delwin(links);
	}
	if (NULL != rechts) {
		delwin(rechts);
	}
	if (NULL != unten) {
		delwin(unten);
	}
	/* curses mode beenden */
	endwin();
	return EXIT_SUCCESS;
}
